package wood.keith.opentools.wizards.jsptags;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.text.MessageFormat;
import java.util.ResourceBundle;
import java.util.SortedMap;
import java.util.TreeMap;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import com.borland.jbuilder.wizard.common.ProjectUtil;
import com.borland.primetime.node.Project;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.wizard.BasicWizardPage;

/**
 * This class contains the wizard page that allows the user to specify
 * the details about the JSP tag.
 *
 * @author   Keith Wood
 * @version  1.0  9 November 2000
 * @version  2.0  6 February 2002 - added JSP 1.2 support
 * @version  3.0  10 February 2004 - added JSP 2.0 support
 */
public class JSPTagWizardPage extends BasicWizardPage {

  private static final ResourceBundle res = Res.getResource();

  // UI variables
	private BorderLayout _borderLayout = new BorderLayout();
	private JPanel _bodyPanel = new JPanel();
	private GridBagLayout _gridBagLayout = new GridBagLayout();
	private JPanel _versionPanel = new JPanel();
	private FlowLayout _flowLayout = new FlowLayout();
	private ButtonGroup _versionGroup = new ButtonGroup();
	private JRadioButton _jsp12Radio = new JRadioButton();
	private JRadioButton _jsp11Radio = new JRadioButton();
	private JLabel _jLabel1 = new JLabel();
	private JComboBox _packageCombo = new JComboBox();
	private JLabel _jLabel2 = new JLabel();
	private JTextField _classNameField = new JTextField();
	private JCheckBox _endCheckBox = new JCheckBox();
	private JCheckBox _nestedCheckBox = new JCheckBox();
	private JCheckBox _bodyCheckBox = new JCheckBox();
	private JCheckBox _repeatCheckBox = new JCheckBox();
	private JCheckBox _tryCheckBox = new JCheckBox();
	private JLabel _jLabel3 = new JLabel();
	private JTextField _propField = new JTextField();
	private JComboBox _propTypeCombo = new JComboBox();
	private JScrollPane _propScroll = new JScrollPane();
	private JTable _propTable = new JTable();
	private JButton _addButton = new JButton();
	private JButton _deleteButton = new JButton();
	private JLabel _jLabel4 = new JLabel();
  private JRadioButton _jsp20Radio = new JRadioButton();
  private JCheckBox _simpleCheckBox = new JCheckBox();
  private JCheckBox _dynamicCheckBox = new JCheckBox();
  private JCheckBox _extraCheckBox = new JCheckBox();

  /**
   * Create a new wizard page.
   * 
   * @param  project  the current project
   */
  public JSPTagWizardPage(Project project) {
    try {
      ProjectUtil.addClassPathText(ProjectUtil.getPackageName(project));
      jbInit();
      _jsp20Radio.setSelected(JSPTagProperties.JSP_VERSION.getValue().
        equals(JSPTagProperties.JSP_20));
      _jsp12Radio.setSelected(JSPTagProperties.JSP_VERSION.getValue().
        equals(JSPTagProperties.JSP_12));
      _jspRadio_actionPerformed(null);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

	/**
	 * Define the user interface.
	 *
	 * The user interface is constructed here so the JBuilder UI Designer
	 * may be used to edit it.
   * 
   * @throws  Exception  if an error arises
	 */
	protected void jbInit() throws Exception {
		this.setPageTitle(res.getString("JSPPageName"));
		try {
			this.setLargeIcon(new ImageIcon(JSPTagWizardPage.class.getResource(
				"/com/borland/jbuilder/wizard/jsp/JspWizardLarge.gif")));
		}
		catch (Exception ex) {
			// Ignore
		}
		this.setInstructions(res.getString("JSPPageInstructions"));
		this.setLayout(_borderLayout);

		_jLabel1.setDisplayedMnemonic(res.getString("FieldPackageChar").charAt(0));
		_jLabel1.setLabelFor(_packageCombo);
		_jLabel1.setText(res.getString("FieldPackageName"));
		_packageCombo.setMaximumSize(new Dimension(32767, 21));
		_packageCombo.setMinimumSize(new Dimension(237, 21));
		_packageCombo.setPreferredSize(new Dimension(241, 21));
		_packageCombo.setToolTipText(res.getString("FieldPackageTip"));
		_packageCombo.setEditable(true);
		_packageCombo.setModel(ProjectUtil.getClassPathHistory());
		_jLabel2.setText(res.getString("FieldClassName"));
		_jLabel2.setDisplayedMnemonic(res.getString("FieldClassChar").charAt(0));
		_jLabel2.setLabelFor(_classNameField);
		_classNameField.setToolTipText(res.getString("FieldClassTip"));
		_endCheckBox.setMargin(new Insets(0, 0, 0, 0));
		_endCheckBox.setText(res.getString("FieldEndCheckboxName"));
		_endCheckBox.setToolTipText(res.getString("FieldEndCheckboxTip"));
		_endCheckBox.setMnemonic(res.getString("FieldEndCheckboxChar").charAt(0));
		_nestedCheckBox.setMargin(new Insets(0, 0, 0, 0));
		_nestedCheckBox.setText(res.getString("FieldNestedCheckboxName"));
		_nestedCheckBox.setToolTipText(res.getString("FieldNestedCheckboxTip"));
		_nestedCheckBox.setMnemonic(res.getString("FieldNestedCheckboxChar").charAt(0));
		_nestedCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
		_bodyCheckBox.setMargin(new Insets(0, 0, 0, 0));
		_bodyCheckBox.setText(res.getString("FieldBodyCheckboxName"));
		_bodyCheckBox.setToolTipText(res.getString("FieldBodyCheckboxTip"));
		_bodyCheckBox.setMnemonic(res.getString("FieldBodyCheckboxChar").charAt(0));
		_bodyCheckBox.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				_bodyCheckBox_itemStateChanged(e);
			}
		});
		_repeatCheckBox.setMargin(new Insets(0, 0, 0, 0));
		_repeatCheckBox.setText(res.getString("FieldRepeatCheckboxName"));
		_repeatCheckBox.setToolTipText(res.getString("FieldRepeatCheckboxTip"));
		_repeatCheckBox.setMnemonic(res.getString("FieldRepeatCheckboxChar").charAt(0));
		_repeatCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
		_repeatCheckBox.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				_bodyCheckBox_itemStateChanged(e);
			}
		});
		_jLabel3.setText(res.getString("FieldPropertiesName"));
		_jLabel3.setLabelFor(_propField);
		_jLabel3.setDisplayedMnemonic(res.getString("FieldPropertiesChar").charAt(0));
		_propField.setMinimumSize(new Dimension(131, 21));
		_propField.setPreferredSize(new Dimension(131, 21));
		_propField.setToolTipText(res.getString("FieldPropertyNameTip"));
		_propField.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
				disEnableButtons();
			}
			public void insertUpdate(DocumentEvent e) {
				disEnableButtons();
			}
			public void removeUpdate(DocumentEvent e) {
				disEnableButtons();
			}
		});
		_propTypeCombo.addItem("String");
		_propTypeCombo.addItem("int");
		_propTypeCombo.addItem("boolean");
		_propTypeCombo.addItem("Date");
		_propTypeCombo.setMaximumSize(new Dimension(32767, 21));
		_propTypeCombo.setMinimumSize(new Dimension(131, 21));
		_propTypeCombo.setPreferredSize(new Dimension(131, 21));
		_propTypeCombo.setToolTipText(res.getString("FieldPropertyTypeTip"));
		_propTable.setMaximumSize(new Dimension(0, 0));
		_propTable.setMinimumSize(new Dimension(0, 0));
		_propTable.setToolTipText(res.getString("FieldPropertiesTip"));
		_propTable.setPreferredScrollableViewportSize(new Dimension(200, 100));
		_propTable.setModel(new DefaultTableModel(new String[] {res.getString("ColumnName"), res.getString("ColumnType")}, 0) {
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		});
		_propTable.getSelectionModel().
			setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		_propTable.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				disEnableButtons();
			}
		});
		_addButton.setToolTipText(res.getString("ButtonAddTip"));
		_addButton.setMargin(new Insets(2, 4, 2, 4));
		_addButton.setMnemonic(res.getString("ButtonAddChar").charAt(0));
		_addButton.setText(res.getString("ButtonAddName"));
		_addButton.setMinimumSize(new Dimension(37, 21));
		_addButton.setMaximumSize(new Dimension(37, 21));
		_addButton.setPreferredSize(new Dimension(37, 21));
		_addButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_addButton_actionPerformed(e);
			}
		});
		_deleteButton.setToolTipText(res.getString("ButtonDeleteTip"));
		_deleteButton.setMargin(new Insets(2, 4, 2, 4));
		_deleteButton.setMnemonic(res.getString("ButtonDeleteChar").charAt(0));
		_deleteButton.setText(res.getString("ButtonDeleteName"));
		_deleteButton.setPreferredSize(new Dimension(51, 21));
		_deleteButton.setMinimumSize(new Dimension(51, 21));
		_deleteButton.setMaximumSize(new Dimension(51, 21));
		_deleteButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_deleteButton_actionPerformed(e);
			}
		});
		_propScroll.setMaximumSize(new Dimension(0, 0));
		_propScroll.setMinimumSize(new Dimension(0, 0));
		_propScroll.setPreferredSize(new Dimension(0, 0));
		_versionPanel.setLayout(_flowLayout);
		_flowLayout.setHgap(20);
		_flowLayout.setVgap(0);
		_jsp12Radio.setMargin(new Insets(0, 0, 0, 0));
		_jsp12Radio.setPreferredSize(new Dimension(68, 21));
		_jsp12Radio.setMinimumSize(new Dimension(68, 21));
		_jsp12Radio.setText("JSP 1.2");
		_jsp12Radio.setToolTipText(res.getString("FieldJSP12Tip"));
		_jsp12Radio.setMnemonic('2');
		_jsp12Radio.setMaximumSize(new Dimension(68, 21));
		_jsp12Radio.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_jspRadio_actionPerformed(e);
			}
		});
		_jsp11Radio.setMargin(new Insets(0, 0, 0, 0));
		_jsp11Radio.setPreferredSize(new Dimension(68, 21));
		_jsp11Radio.setMinimumSize(new Dimension(68, 21));
		_jsp11Radio.setText("JSP 1.1");
		_jsp11Radio.setToolTipText(res.getString("FieldJSP11Tip"));
		_jsp11Radio.setMnemonic('1');
		_jsp11Radio.setMaximumSize(new Dimension(68, 21));
		_jsp11Radio.setSelected(true);
		_jsp11Radio.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_jspRadio_actionPerformed(e);
			}
		});
		_tryCheckBox.setMargin(new Insets(0, 0, 0, 0));
		_tryCheckBox.setEnabled(false);
		_tryCheckBox.setText(res.getString("FieldTryCheckboxName"));
		_tryCheckBox.setToolTipText(res.getString("FieldTryCheckboxTip"));
		_tryCheckBox.setMnemonic(res.getString("FieldTryCheckboxChar").charAt(0));
		_jLabel4.setText(res.getString("FieldOptionsName"));
    _jsp20Radio.setMargin(new Insets(0, 0, 0, 0));
    _jsp20Radio.setPreferredSize(new Dimension(68, 21));
    _jsp20Radio.setMinimumSize(new Dimension(68, 21));
    _jsp20Radio.setToolTipText(res.getString("FieldJSP20Tip"));
    _jsp20Radio.setMaximumSize(new Dimension(68, 21));
		_jsp20Radio.setMnemonic('0');
    _jsp20Radio.setText("JSP 2.0");
    _jsp20Radio.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _jspRadio_actionPerformed(e);
      }
    });
    _simpleCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    _simpleCheckBox.setMnemonic(res.getString("FieldSimpleCheckboxChar").charAt(0));
    _simpleCheckBox.setEnabled(false);
    _simpleCheckBox.setToolTipText(res.getString("FieldSimpleCheckboxTip"));
    _simpleCheckBox.setText(res.getString("FieldSimpleCheckboxName"));
    _simpleCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _simpleCheckBox.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _simpleCheckBox_itemStateChanged(e);
      }
    });
    _dynamicCheckBox.setEnabled(false);
    _dynamicCheckBox.setMaximumSize(new Dimension(81, 18));
    _dynamicCheckBox.setMinimumSize(new Dimension(81, 18));
    _dynamicCheckBox.setPreferredSize(new Dimension(150, 18));
    _dynamicCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _dynamicCheckBox.setMnemonic(res.getString("FieldDynamicCheckboxChar").charAt(0));
    _dynamicCheckBox.setText(res.getString("FieldDynamicCheckboxName"));
    _dynamicCheckBox.setToolTipText(res.getString("FieldDynamicCheckboxTip"));
    _extraCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    _extraCheckBox.setMnemonic(res.getString("FieldExtraCheckboxChar").charAt(0));
    _extraCheckBox.setToolTipText(res.getString("FieldExtraCheckboxTip"));
    _extraCheckBox.setText(res.getString("FieldExtraCheckboxName"));
    _extraCheckBox.setMargin(new Insets(0, 0, 0, 0));
    this.add(_bodyPanel, BorderLayout.CENTER);
		_bodyPanel.setLayout(_gridBagLayout);

    _bodyPanel.add(_versionPanel,        new GridBagConstraints(0, 0, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _versionPanel.add(_jsp11Radio, null);
    _versionPanel.add(_jsp12Radio, null);
    _versionPanel.add(_jsp20Radio, null);
		_bodyPanel.add(_jLabel1,        new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_packageCombo,         new GridBagConstraints(1, 1, 3, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 2, 0), 0, 0));
        _bodyPanel.add(_jLabel2,        new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_classNameField,         new GridBagConstraints(1, 2, 3, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 0, 0), 0, 0));
    _bodyPanel.add(_jLabel4,        new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_endCheckBox,          new GridBagConstraints(1, 3, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 2, 0, 0), 0, 0));
		_bodyPanel.add(_nestedCheckBox,         new GridBagConstraints(1, 3, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 4, 0, 0), 0, 0));
		_bodyPanel.add(_bodyCheckBox,            new GridBagConstraints(1, 4, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 2, 0, 0), 0, 0));
		_bodyPanel.add(_repeatCheckBox,         new GridBagConstraints(1, 4, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 4, 0, 0), 0, 0));
    _bodyPanel.add(_tryCheckBox,            new GridBagConstraints(1, 5, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 2, 0, 0), 0, 0));
    _bodyPanel.add(_simpleCheckBox,   new GridBagConstraints(1, 5, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 4, 0, 0), 0, 0));
    _bodyPanel.add(_dynamicCheckBox,            new GridBagConstraints(1, 6, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 2, 0, 0), 0, 0));
    _bodyPanel.add(_extraCheckBox,   new GridBagConstraints(1, 6, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 4, 0, 0), 0, 0));
		_bodyPanel.add(_jLabel3,        new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_propField,        new GridBagConstraints(1, 7, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
		_bodyPanel.add(_propTypeCombo,        new GridBagConstraints(2, 7, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 2, 2), 0, 0));
		_bodyPanel.add(_propScroll,         new GridBagConstraints(1, 8, 2, 1, 0.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 0, 2), 0, 0));
		_propScroll.getViewport().add(_propTable, null);
		_bodyPanel.add(_addButton,        new GridBagConstraints(3, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_deleteButton,        new GridBagConstraints(3, 8, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
		disEnableButtons();
		_versionGroup.add(_jsp11Radio);
		_versionGroup.add(_jsp12Radio);
    _versionGroup.add(_jsp20Radio);
	}

	/**
	 * Dis/enable other options based on specification in use.
   * 
   * @param  e  the triggering event
	 */
	void _jspRadio_actionPerformed(ActionEvent e) {
		_tryCheckBox.setEnabled(!_jsp11Radio.isSelected());
    _simpleCheckBox.setEnabled(_jsp20Radio.isSelected());
    _dynamicCheckBox.setEnabled(_jsp20Radio.isSelected());
	}

	/**
	 * Force manipulate body option (but only under JSP 1.1).
   * 
   * @param  e  the triggering event
	 */
	void _bodyCheckBox_itemStateChanged(ItemEvent e) {
		if (_repeatCheckBox.isSelected() && !_bodyCheckBox.isSelected() &&
				_jsp11Radio.isSelected()) {
			_bodyCheckBox.setSelected(true);
		}
	}

  /**
   * Simple tags don't worry about other options.
   * 
   * @param  e  the triggering event
   */
  void _simpleCheckBox_itemStateChanged(ItemEvent e) {
    boolean isSimple = _simpleCheckBox.isSelected();
    _endCheckBox.setEnabled(!isSimple);
    if (isSimple) {
      _endCheckBox.setSelected(false);
    }
  }

	/**
	 * Add a new property to the list.
   * 
   * @param  e  the triggering event
	 */
	void _addButton_actionPerformed(ActionEvent e) {
		if (_propField.getText().equals("id") ||
				_propField.getText().equals("parent") ||
				_propField.getText().equals("value")) {
			JOptionPane.showMessageDialog(wizardHost.getDialogParent(),
				MessageFormat.format(res.getString("AddButtonWarning"), 
        new Object[] {_propField.getText()}),
				JSPTagWizard.TITLE, JOptionPane.WARNING_MESSAGE, null);
		}
		DefaultTableModel model = (DefaultTableModel)_propTable.getModel();
		model.addRow(new Object[]
			{_propField.getText(), _propTypeCombo.getSelectedItem()});
		_propField.setText("");
		_propField.requestFocus();
		disEnableButtons();
	}

	/**
	 * Delete a property from the list.
   * 
   * @param  e  the triggering event
	 */
	void _deleteButton_actionPerformed(ActionEvent e) {
		DefaultTableModel model = (DefaultTableModel)_propTable.getModel();
		model.removeRow(_propTable.getSelectedRow());
		_propField.requestFocus();
		disEnableButtons();
	}

	/**
	 * Synchronise buttons with field contents.
	 */
	private void disEnableButtons() {
		DefaultTableModel model = (DefaultTableModel)_propTable.getModel();
		boolean present = false;
		for (int index = 0; index < model.getRowCount(); index++) {
			if (model.getValueAt(index, 0).equals(_propField.getText())) {
				present = true;
				break;
			}
		}
		_addButton.setEnabled((_propField.getText().length() != 0) && !present);
		_deleteButton.setEnabled(_propTable.getSelectedRow() > -1);
	}

	/**
	 * Check for user input error.
	 *
	 * The user has completed this page so check for input error.  If any, then
	 * report the error in a dialog and throw an exception.
   * 
   * @throws  VetoException  if the page is invalid
	 */
	public void checkPage() throws VetoException {
		// File name must be specified
		if (getClassName().length() == 0) {
			_classNameField.requestFocus();
			JOptionPane.showMessageDialog(wizardHost.getDialogParent(),
				res.getString("ClassNameMandatory"),
				JSPTagWizard.TITLE, JOptionPane.ERROR_MESSAGE, null);
			throw new VetoException();
		}
	}

	/**
	 * Fetches the package name for the tag class.
	 *
	 * @return  the package name
	 */
	public String getPackageName() {
		return _packageCombo.getEditor().getItem().toString();
	}

	/**
	 * Fetches the class name for the tag class.
	 *
	 * @return  the new class name
	 */
	public String getClassName() {
		return _classNameField.getText();
	}

	/**
	 * Fetches the JSP version selected.
	 *
	 * @return  the JSP version
	 */
	public String getJSPVersion() {
		return (_jsp20Radio.isSelected() ? JSPTagProperties.JSP_20 :
        (_jsp12Radio.isSelected() ? JSPTagProperties.JSP_12 : 
        JSPTagProperties.JSP_11));
	}

	/**
	 * Is the tag post-processed.
	 *
	 * @return  true if the tag adds stuff after the body, false otherwise
	 */
	public boolean isEndUsed() {
		return _endCheckBox.isSelected();
	}

	/**
	 * Is the tag nested.
	 *
	 * @return  true if the tag refers to a surrounding tag, false otherwise
	 */
	public boolean isNested() {
		return _nestedCheckBox.isSelected();
	}

	/**
	 * Does the tag alter its content.
	 *
	 * @return  true if the tag alters its body, false otherwise
	 */
	public boolean isBodyAltered() {
		return _bodyCheckBox.isSelected();
	}

	/**
	 * Does the tag repeatedly process its body.
	 *
	 * @return  true if the tag loops over the body, false otherwise
	 */
	public boolean isRepeated() {
		return _repeatCheckBox.isSelected();
	}

	/**
	 * Does the tag implement TryCatchFinally.
	 *
	 * @return  true if the tag implements this interface, false otherwise
	 */
	public boolean isTryCatchFinally() {
		return !_jsp11Radio.isSelected() && _tryCheckBox.isSelected();
	}

  /**
   * Does the tag have a simple life cycle.
   *
   * @return  true if the tag is simple, false otherwise
   */
  public boolean isSimpleTag() {
    return _jsp20Radio.isSelected() && _simpleCheckBox.isSelected();
  }

  /**
   * Does the tag implement DynamicAttributes.
   *
   * @return  true if the tag implements this interface, false otherwise
   */
  public boolean isDynamicAttributes() {
    return _jsp20Radio.isSelected() && _dynamicCheckBox.isSelected();
  }

  /**
   * Does the tag supply extra information.
   *
   * @return  true if the tag provides extra info, false otherwise
   */
  public boolean isExtraInfo() {
    return _extraCheckBox.isSelected();
  }

	/**
	 * Fetches the list of properties and their types for the tag.
	 *
	 * @return  the property list
	 */
	public SortedMap getProps() {
		TableModel model = _propTable.getModel();
		TreeMap props = new TreeMap();
		for (int index = 0; index < model.getRowCount(); index++) {
			props.put(model.getValueAt(index, 0), model.getValueAt(index, 1));
		}
		return props;
	}
}
