package wood.keith.opentools.wizards.jsptags;

/**
 * Encapsulate details about a tag attribute within a JSP tag library.
 *
 * @author   Keith Wood
 * @version  1.0  9 November 2000
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorAttribute extends TaglibDescriptorElement {

	// Internal variables
  private String _name;
	private boolean _included = true;
	private boolean _required;
	private boolean _expression = true;
  private boolean _fragment;
	private String _type;

	/**
	 * Build a new tag attribute.
	 *
	 * @param  name  the name of the attribute in a tag
   * @param  type  the type of the attribute
	 */
	public TaglibDescriptorAttribute(String name, String type) {
		super();
		setName(name);
    setType(type);
	}

	private void setName(String value) { _name = value; }

	public String getName() { return _name; }

	public void setIncluded(boolean value) { _included = value; }

	public boolean isIncluded() { return _included; }

	public void setRequired(boolean value) { _required = value; }

	public boolean isRequired() { return _required; }

	public void setExpressionAllowed(boolean value) { _expression = value; }

	public boolean isExpressionAllowed() { return _expression; }

  public void setFragment(boolean value) { _fragment = value; }

  public boolean isFragment() { return _fragment; }

	public void setType(String value) { _type = value; }

	public String getType() { return _type; }

	public String toString() { return getName(); }

	/**
	 * Format the attribute for inclusion in a taglib descriptor file.
	 *
	 * @param  jspVersion  the JSP version in use
	 * @return  the attribute definition in XML
	 */
	public String toXml(String jspVersion) {
		if (!isIncluded()) {
			return "";
		}
		StringBuffer buffer = new StringBuffer("    <attribute>\n");
    if (jspVersion.equals(JSPTagProperties.JSP_20)) {
      addXmlElement(buffer, "description", getDescription(), "      ");
    }
		addXmlElement(buffer, "name", getName(), "      ");
		addXmlElement(buffer, "required", String.valueOf(isRequired()), "      ");
    if (jspVersion.equals(JSPTagProperties.JSP_20) && isFragment()) {
      addXmlElement(buffer, "fragment", String.valueOf(isFragment()), "      ");
    }
    else {
  		addXmlElement(buffer, "rtexprvalue", 
          String.valueOf(isExpressionAllowed()), "      ");
		  if (!jspVersion.equals(JSPTagProperties.JSP_11)) {
        addXmlElement(buffer, "type", getType(), "      ");
      }
    }
    if (jspVersion.equals(JSPTagProperties.JSP_12)) {
			addXmlElement(buffer, "description", getDescription(), "      ");
		}
		buffer.append("    </attribute>\n");
		return buffer.toString();
	}
}
