package wood.keith.opentools.wizards.jsptags;

import java.util.ArrayList;

/**
 * Encapsulate details about an element within a JSP tag library.
 *
 * @author   Keith Wood
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public abstract class TaglibDescriptorElement {

	// Internal variables
  protected String _description = "";
	protected String _displayName = "";
  protected String _smallIcon = "";
  protected String _largeIcon = "";

  public void setDescription(String value) { _description = value; }

  public String getDescription() { return _description; }

	public void setDisplayName(String value) { _displayName = value; }

	public String getDisplayName() { return _displayName; }

  public void setSmallIcon(String value) { _smallIcon = value; }

  public String getSmallIcon() { return _smallIcon; }

  public void setLargeIcon(String value) { _largeIcon = value; }

  public String getLargeIcon() { return _largeIcon; }

	/**
	 * Format the taglib element for inclusion in a taglib descriptor file.
	 *
   * @param  jspVersion  the version of JSP in use
	 * @return  the tag file definition in XML
	 */
	public abstract String toXml(String jspVersion);

  /**
   * Generate the standard description group sub-elements.
   *
   * @param  buffer  the string buffer to add to
   * @param  indent  the indentation spacing
   */
  protected void addDescriptionGroup(StringBuffer buffer, String indent) {
    addXmlElement(buffer, "description", getDescription(), indent);
    addXmlElement(buffer, "display-name", getDisplayName(), indent);
    if (getSmallIcon().length() > 0 || getLargeIcon().length() > 0) {
      buffer.append(indent).append("<icon>\n");
      addXmlElement(buffer, "small-icon", getSmallIcon(), indent + "  ");
      addXmlElement(buffer, "large-icon", getLargeIcon(), indent + "  ");
      buffer.append(indent).append("</icon>\n");
    }
  }

  /**
   * Generate XML elements for the tag library.
   *
   * @param  buffer      the string buffer to add to
   * @param  jspVersion  the version of JSP in use
   * @param  items       the list of tag library items to convert to XML
   */
  protected void addXmlElements(StringBuffer buffer, String jspVersion, 
      ArrayList items) {
    if (items.size() == 0) {
      return;
    }
    for (int index = 0; index < items.size(); index++) {
      buffer.append(
          ((TaglibDescriptorElement)items.get(index)).toXml(jspVersion));
    }
  }

  /**
   * Generate a simple XML element with text content.
   *
   * @param  buffer  the string buffer to add to
   * @param  name    the name of the element
   * @param  text    the textual content
   * @param  indent  the indentation spacing
   */
  protected void addXmlElement(StringBuffer buffer, String name, String text, 
      String indent) {
    addXmlElement(buffer, name, text, indent, false);
  }

  /**
   * Generate a simple XML element with text content.
   *
   * @param  buffer    the string buffer to add to
   * @param  name      the name of the element
   * @param  text      the textual content
   * @param  indent    the indentation spacing
   * @param  required  true if this element must appear, 
   *                   false if it can be omitted if the text value is blank
   */
  protected void addXmlElement(StringBuffer buffer, String name, String text, 
      String indent, boolean required) {
    if (required || (text != null && text.length() > 0)) {
      boolean asCDATA = (text.indexOf("<") > -1 || text.indexOf(">") > -1 || 
          text.indexOf("&") > -1);
      buffer.append(indent).append("<").append(name).append(">").
          append(asCDATA ? "<![CDATA[" : "").append(text).
          append(asCDATA ? "]]>" : "").append("</").append(name).append(">\n");
    }
  }
}
