package wood.keith.opentools.wizards.jsptags;

import java.util.ArrayList;

/**
 * Encapsulate details about a validator parameter within a JSP tag library.
 *
 * @author   Keith Wood
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorParameter extends TaglibDescriptorElement {

	// Internal variables
  private String _value;

  /**
   * Create a new JSP paramater item.
   * 
   * @param  name         the name of the parameter
   * @param  value        its value
   * @param  description  its description
   */
	public TaglibDescriptorParameter(String name, String value, 
      String description) {
		setDisplayName(name);
    _value = value;
    setDescription(description);
	}

  public void setValue(String value) { _value = value; }

	public String getValue() { return _value; }

	public String toString() { return getDisplayName(); }

  /**
   * Format the parameter for inclusion in a taglib descriptor file.
   *
   * @param  jspVersion  the version of JSP in use
   * @return  the parameter definition in XML
   */
  public String toXml(String jspVersion) {
    StringBuffer buffer = new StringBuffer("    <init-param>\n");
    if (jspVersion.equals(JSPTagProperties.JSP_20)) {
      addXmlElement(buffer, "description", getDescription(), "      ");
    }
    addXmlElement(buffer, "param-name", getDisplayName(), "      ");
    addXmlElement(buffer, "param-value", getValue(), "      ", true);
    if (jspVersion.equals(JSPTagProperties.JSP_12)) {
      addXmlElement(buffer, "description", getDescription(), "      ");
    }
    buffer.append("    </init-param>\n");
    return buffer.toString();
  }
}
