package wood.keith.opentools.wizards.jsptags;

import java.util.ArrayList;

/**
 * Encapsulate details about a validator within a JSP tag library.
 *
 * @author   Keith Wood
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorValidator extends TaglibDescriptorElement {

	// Internal variables
  private String _className;
	private ArrayList _parameters = new ArrayList();

  /**
   * Create a new JSP validator item.
   * 
   * @param  className    the name of the validator class
   * @param  description  its description
   */
	public TaglibDescriptorValidator(String className, String description) {
		_className = className;
    setDescription(description);
	}

	public String getClassName() { return _className; }

	/**
	 * Provide access to the parameters for this validator.
   * Each entry is a TaglibDescriptorParameter.
	 *
	 * @return  a list of parameter details
	 */
	public ArrayList getParameters() { return _parameters; }

	public String toString() { return getClassName(); }

  /**
   * Format the validator for inclusion in a taglib descriptor file.
   *
   * @param  jspVersion  the version of JSP in use
   * @return  the validator definition in XML
   */
  public String toXml(String jspVersion) {
    StringBuffer buffer = new StringBuffer("\n  <validator>\n");
    if (jspVersion.equals(JSPTagProperties.JSP_20)) {
      addXmlElement(buffer, "description", getDescription(), "    ");
    }
    addXmlElement(buffer, "validator-class", getClassName(), "    ");
    addXmlElements(buffer, jspVersion, getParameters());
    if (jspVersion.equals(JSPTagProperties.JSP_12)) {
      addXmlElement(buffer, "description", getDescription(), "    ");
    }
    buffer.append("  </validator>\n");
    return buffer.toString();
  }
}
