package wood.keith.opentools.wizards.jsptags;

import java.util.Arrays;

/**
 * Encapsulate details about a tag variable within a JSP tag library.
 *
 * @author   Keith Wood
 * @version  1.0  6 February 2002
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorVariable extends TaglibDescriptorElement {
  
  /** Possible scope values for a variable. */
  public static final String[] SCOPE_VALUES = {"AT_BEGIN", "AT_END", "NESTED"};

	// Internal variables
	private String _name;
	private boolean _fromAttribute;
	private String _className = "java.lang.String";
	private boolean _declared = true;
	private String _scope = SCOPE_VALUES[2];

	/**
	 * Build a new tag variable.
	 *
	 * @param  name           the name of the variable in a tag
	 * @param  fromAttribute  is this name the name of an attribute?
	 */
	public TaglibDescriptorVariable(String name, boolean fromAttribute) {
		super();
		setName(name);
		setFromAttribute(fromAttribute);
	}

	private void setName(String value) { _name = value; }

	public String getName() { return _name; }

	public void setFromAttribute(boolean value) { _fromAttribute = value; }

	public boolean isFromAttribute() { return _fromAttribute; }

	public void setClassName(String value) { _className = value; }

	public String getClassName() { return _className; }

	public void setDeclared(boolean value) { _declared = value; }

	public boolean isDeclared() { return _declared; }

	public void setScope(String value) {
		if (Arrays.binarySearch(SCOPE_VALUES, value) > -1) {
			_scope = value;
		}
	}

	public String getScope() { return _scope; }

	public String toString() { return getName(); }

	/**
	 * Format the variable for inclusion in a taglib descriptor file.
	 *
   * @param  jspVersion  the version of JSP in use
	 * @return  the variable definition in XML
	 */
	public String toXml(String jspVersion) {
		StringBuffer buffer = new StringBuffer("    <variable>\n");
    if (jspVersion.equals(JSPTagProperties.JSP_20)) {
      addXmlElement(buffer, "description", getDescription(), "      ");
    }
		addXmlElement(buffer, (_fromAttribute ? 
        "name-from-attribute" : "name-given"), getName(), "      ");
		addXmlElement(buffer, "variable-class", getClassName(), "      ");
		addXmlElement(buffer, "declare", String.valueOf(isDeclared()), "      ");
    addXmlElement(buffer, "scope", getScope(), "      ");
		if (jspVersion.equals(JSPTagProperties.JSP_12)) {
			addXmlElement(buffer, "description", getDescription(), "      ");
		}
		buffer.append("    </variable>\n");
		return buffer.toString();
	}
}
