package wood.keith.opentools.wizards.jsptags;

import java.awt.Color;
import java.util.ResourceBundle;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.borland.primetime.util.VetoException;
import com.borland.primetime.wizard.BasicWizardPage;
import com.borland.primetime.wizard.WizardHost;

/**
 * This class contains the common abilities of the wizard pages.
 *
 * @author   Keith Wood
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorWizardPage extends BasicWizardPage {

  protected static final ResourceBundle res = Res.getResource();
  protected static final String NO_ENTRY = res.getString("None");

  // Internal variables
  protected TaglibDescriptorWizard _wizard;
  protected int _pageNum;
  protected String _jspVersion = JSPTagProperties.JSP_11;
  protected Color _enabledBackground;
  protected Color _disabledBackground;
  protected Icon _icon = new ImageIcon(this.getClass().getResource(
      "/com/borland/jbuilder/wizard/jsp/JspWizardLarge.gif"));
  
  public TaglibDescriptorWizardPage() {}

  /**
   * Create a new wizard page.
   *
   * @param  wizard   the wizard invoking this page
   * @param  pageNum  the number of this page within the wizard
   */
  public TaglibDescriptorWizardPage(TaglibDescriptorWizard wizard, 
      int pageNum) {
    _wizard = wizard;
    _pageNum = pageNum;
    _enabledBackground = new JTextField().getBackground();
    _disabledBackground = new JPanel().getBackground();
  }

	/**
	 * Update UI for JSP version on activation.
	 *
	 * @param  host  the hosting dialog
	 */
	public void activated(WizardHost host) {
		super.activated(host);
    host.setFinishEnabled(true);
    setJSPVersion(_wizard.getJSPVersion());
	}

  /**
   * Setup based on JSP version.
   *
   * @param  jspVersion  the version of JSP in use
   */
  protected void setJSPVersion(String jspVersion) {
    _jspVersion = jspVersion;
    updateControls();
  }

  /**
   * Return the JSP version.
   *
   * @return  the version of JSP in use
   */
  protected String getJSPVersion() { return _jspVersion; }
  
  /**
   * Return the number of this page within the wizard.
   * 
   * @return  the current page number
   */
  public int getPageNum() { return _pageNum; }
  
  /**
   * Synchronise controls with field contents and JSP version.
   */
  protected void updateControls() {}

	/**
	 * Find enabled/disabled colours.
	 *
	 * @param  enabled  true for enabled, false for disabled
	 * @return  the background colour
	 */
	protected Color getFieldBackground(boolean enabled) {
		return (enabled ? _enabledBackground : _disabledBackground);
	}

	/**
	 * Check for a user input error.
	 *
	 * @param  condition  a test that must be true for the page to be valid
	 * @param  message    the text to display
	 * @param  focus      the UI component to move focus to
	 * @throws  VetoException  if the condition is not met
	 */
	protected void checkCondition(boolean condition, String message,
			JComponent focus) throws VetoException {
		if (!condition) {
			focus.requestFocus();
			JOptionPane.showMessageDialog(wizardHost.getDialogParent(), message,
				TaglibDescriptorWizard.TITLE, JOptionPane.ERROR_MESSAGE, null);
			throw new VetoException();
		}
	}
}
