package wood.keith.opentools.wizards.jsptags;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.io.File;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.filechooser.FileFilter;

import com.borland.primetime.util.VetoException;

/**
 * This class contains the wizard page which allows the user to specify
 * the details about the taglib as a whole.
 *
 * @author   Keith Wood
 * @version  1.0  9 November 2000
 * @version  2.0  6 February 2002 - JSP 1.2
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorWizardPage1 extends TaglibDescriptorWizardPage {

  private static final String TLD_FILE_EXT = ".tld";
  private static final String TLD_FILE_DESC = res.getString("TLDFilesName");

  // Internal variables
  private String _shortName;

  private BorderLayout _borderLayout = new BorderLayout();
  private JPanel _bodyPanel = new JPanel();
  private GridBagLayout _gridBagLayout = new GridBagLayout();
  private JLabel _jLabel4 = new JLabel();
  private JPanel _versionPanel = new JPanel();
  private FlowLayout _flowLayout = new FlowLayout();
  private ButtonGroup _versionGroup = new ButtonGroup();
  private JRadioButton _jsp12Radio = new JRadioButton();
  private JRadioButton _jsp11Radio = new JRadioButton();
  private JLabel _jLabel1 = new JLabel();
  private JTextField _shortNameText = new JTextField();
  private JLabel _displayNameLabel = new JLabel();
  private JTextField _displayNameText = new JTextField();
  private JLabel _jLabel2 = new JLabel();
  private JTextField _uriText = new JTextField();
  private JLabel _smallIconLabel = new JLabel();
  private JTextField _smallIconText = new JTextField();
  private JLabel _largeIconLabel = new JLabel();
  private JTextField _largeIconText = new JTextField();
  private JLabel _jLabel3 = new JLabel();
  private JScrollPane _infoScroll = new JScrollPane();
  private JTextArea _infoText = new JTextArea();
  private Border _textBorder;
  private JLabel _jLabel5 = new JLabel();
  private JTextField _fileField = new JTextField();
  private JButton _browseButton = new JButton();
  private JRadioButton _jsp20Radio = new JRadioButton();

  /**
   * Create a new wizard page.
   *
   * @param  wizard     the wizard invoking this page
   * @param  shortName  the initial value for the short name of the tag library,
   *                    if null then no tags were found and only a message
   *                    is displayed
   * @param  fileName   the initial file name for the TLD
   */
  public TaglibDescriptorWizardPage1(TaglibDescriptorWizard wizard,
      String shortName, String fileName) {
    super(wizard, 1);
    try {
      _shortName = shortName;
      jbInit();
      initialize(fileName);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**
   * Define the user interface.
   *
   * The user interface is constructed here so the JBuilder UI Designer
   * may be used to edit it.
   *
   * @throws  Exception
   */
  protected void jbInit() throws Exception {
    _textBorder = BorderFactory.createEtchedBorder(SystemColor.controlLtHighlight,SystemColor.controlShadow);
    this.setPageTitle(res.getString("TLDPage1Title"));
    try {
      this.setLargeIcon(_icon);
    }
    catch (Exception ex) {
      // Ignore
    }
    this.setInstructions(res.getString("TLDPage1Instructions"));
    this.setLayout(_borderLayout);

    _jLabel4.setText(res.getString("FieldVersionName"));
    _versionPanel.setMinimumSize(new Dimension(223, 21));
    _versionPanel.setPreferredSize(new Dimension(223, 21));
    _versionPanel.setLayout(_flowLayout);
    _jsp11Radio.setPreferredSize(new Dimension(68, 21));
    _jsp11Radio.setMinimumSize(new Dimension(68, 21));
    _jsp11Radio.setSelected(true);
    _jsp11Radio.setText("JSP 1.1");
    _jsp11Radio.setMnemonic('1');
    _jsp11Radio.setToolTipText(res.getString("FieldJSP11Tip"));
    _jsp11Radio.setActionCommand("jsp11Radio");
    _jsp11Radio.setMaximumSize(new Dimension(104, 21));
    _jsp11Radio.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _jspRadio_actionPerformed(e);
      }
    });
    _jsp12Radio.setPreferredSize(new Dimension(68, 21));
    _jsp12Radio.setMinimumSize(new Dimension(68, 21));
    _jsp12Radio.setText("JSP 1.2");
    _jsp12Radio.setMnemonic('2');
    _jsp12Radio.setToolTipText(res.getString("FieldJSP12Tip"));
    _jsp12Radio.setActionCommand("jsp12Radio");
    _jsp12Radio.setMaximumSize(new Dimension(104, 21));
    _jsp12Radio.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _jspRadio_actionPerformed(e);
      }
    });
    _jsp20Radio.setPreferredSize(new Dimension(68, 21));
    _jsp20Radio.setMinimumSize(new Dimension(68, 21));
    _jsp20Radio.setText("JSP 2.0");
    _jsp20Radio.setMnemonic('0');
    _jsp20Radio.setToolTipText(res.getString("FieldJSP20Tip"));
    _jsp20Radio.setActionCommand("jsp20Radio");
    _jsp20Radio.setMaximumSize(new Dimension(104, 21));
    _jsp20Radio.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _jspRadio_actionPerformed(e);
      }
    });
    _displayNameLabel.setDisplayedMnemonic(res.getString("FieldDisplayNameChar").charAt(0));
    _displayNameLabel.setLabelFor(_displayNameText);
    _displayNameLabel.setText(res.getString("FieldDisplayNameName"));
    _displayNameText.setToolTipText(res.getString("FieldDisplayNameTip"));
    _jLabel1.setText(res.getString("FieldShortNameName"));
    _jLabel1.setDisplayedMnemonic(res.getString("FieldShortNameChar").charAt(0));
    _jLabel1.setLabelFor(_shortNameText);
    _shortNameText.setToolTipText(res.getString("FieldShortNameTip"));
    _jLabel2.setText(res.getString("FieldURIName"));
    _jLabel2.setDisplayedMnemonic(res.getString("FieldURIChar").charAt(0));
    _jLabel2.setLabelFor(_uriText);
    _uriText.setToolTipText(res.getString("FieldURITip"));
    _smallIconLabel.setDisplayedMnemonic(res.getString("FieldSmallIconChar").charAt(0));
    _smallIconLabel.setLabelFor(_smallIconText);
    _smallIconLabel.setText(res.getString("FieldSmallIconName"));
    _smallIconText.setToolTipText(res.getString("FieldSmallIconTip"));
    _largeIconLabel.setDisplayedMnemonic(res.getString("FieldLargeIconChar").charAt(0));
    _largeIconLabel.setLabelFor(_largeIconText);
    _largeIconLabel.setText(res.getString("FieldLargeIconName"));
    _largeIconText.setToolTipText(res.getString("FieldLargeIconTip"));
    _jLabel3.setText(res.getString("FieldDescriptionName"));
    _jLabel3.setDisplayedMnemonic(res.getString("FieldDescriptionChar").charAt(0));
    _jLabel3.setLabelFor(_infoText);
    _infoScroll.setBorder(_textBorder);
    _infoText.setToolTipText(res.getString("FieldDescriptionTip"));
    _infoText.setFont(new java.awt.Font("SansSerif", 0, 12));
    _flowLayout.setHgap(20);
    _flowLayout.setVgap(0);
    _jLabel5.setDisplayedMnemonic(res.getString("FieldFileNameChar").charAt(0));
    _jLabel5.setLabelFor(_fileField);
    _jLabel5.setText(res.getString("FieldFileNameName"));
    _browseButton.setMinimumSize(new Dimension(51, 21));
    _browseButton.setPreferredSize(new Dimension(51, 21));
    _browseButton.setToolTipText(res.getString("ButtonFileNameTip"));
    _browseButton.setMargin(new Insets(2, 2, 2, 2));
    _browseButton.setText(res.getString("ButtonFileNameName"));
    _browseButton.setMnemonic(res.getString("ButtonFileNameChar").charAt(0));
    _browseButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _browseButton_actionPerformed(e);
      }
    });
    _fileField.setToolTipText(res.getString("FieldFileNameTip"));
    this.add(_bodyPanel, BorderLayout.CENTER);
    _bodyPanel.setLayout(_gridBagLayout);

    _bodyPanel.add(_jLabel5,   new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_fileField,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_browseButton,   new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 0, 2, 2), 0, 0));
    _bodyPanel.add(_jLabel4,    new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_versionPanel,   new GridBagConstraints(1, 1, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _versionPanel.add(_jsp11Radio, null);
    _versionPanel.add(_jsp12Radio, null);
    _versionPanel.add(_jsp20Radio, null);
    _bodyPanel.add(_jLabel1,    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_shortNameText,     new GridBagConstraints(1, 2, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_displayNameLabel,    new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_displayNameText,    new GridBagConstraints(1, 3, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_jLabel2,    new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_uriText,     new GridBagConstraints(1, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_smallIconLabel,    new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_smallIconText,    new GridBagConstraints(1, 5, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_largeIconLabel,    new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_largeIconText,    new GridBagConstraints(1, 6, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_jLabel3,    new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    _bodyPanel.add(_infoScroll,    new GridBagConstraints(1, 7, 2, 1, 0.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
    _infoScroll.getViewport().add(_infoText, null);
    _versionGroup.add(_jsp11Radio);
    _versionGroup.add(_jsp12Radio);
    _versionGroup.add(_jsp20Radio);
  }

  /**
   * Initialise UI based on presence of short name.
   * If short name is null then no tags were found.
   *
   * @param  fileName   the initial file name for the TLD
   */
  private void initialize(String fileName) {
    if (_shortName == null) {
      this.setInstructions(res.getString("NoTagsFoundInstructions"));
      _bodyPanel.setVisible(false);
    }
    else {
      _fileField.setText(fileName);
      _jsp20Radio.setSelected(JSPTagProperties.JSP_VERSION.getValue().
        equals(JSPTagProperties.JSP_20));
      _jsp12Radio.setSelected(JSPTagProperties.JSP_VERSION.getValue().
        equals(JSPTagProperties.JSP_12));
      _shortNameText.setText(_shortName);
    }
  }

  /**
   * Update the JSP version in use.
   *
   * @param  e  the triggering event
   */
  void _jspRadio_actionPerformed(ActionEvent e) {
    setJSPVersion(_jsp20Radio.isSelected() ? JSPTagProperties.JSP_20 :
        (_jsp12Radio.isSelected() ? JSPTagProperties.JSP_12 :
        JSPTagProperties.JSP_11));
  }

  /**
   * Select a destination TLD file.
   *
   * @param  e  the triggering event
   */
  void _browseButton_actionPerformed(ActionEvent e) {
    JFileChooser chooser = new JFileChooser(_fileField.getText());
    chooser.setDialogTitle(getPageTitle());
    chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    // Set filters for all and .tag/.tagx files, with the latter as the default
    chooser.addChoosableFileFilter(chooser.getAcceptAllFileFilter());
    chooser.setFileFilter(new FileFilter() {
      public boolean accept(File file) {
        String path = file.getPath();
        return (file.isDirectory() ||
            ((path.length() > TLD_FILE_EXT.length()) &&
            (path.substring(path.length() - TLD_FILE_EXT.length()).
            toLowerCase().equals(TLD_FILE_EXT))));
      }
      public String getDescription() {
        return TLD_FILE_DESC;
      }
    });
    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      _fileField.setText(chooser.getSelectedFile().getAbsolutePath());
    }
  }

  /**
   * Synchronise controls with field contents and JSP version.
   */
  protected void updateControls() {
    boolean isJSP11 = _jspVersion.equals(JSPTagProperties.JSP_11);
    _wizard.setJSPVersion(_jspVersion);
    _displayNameLabel.setEnabled(!isJSP11);
    _displayNameText.setEnabled(!isJSP11);
    _displayNameText.setBackground(getFieldBackground(!isJSP11));
    _smallIconLabel.setEnabled(!isJSP11);
    _smallIconText.setEnabled(!isJSP11);
    _smallIconText.setBackground(getFieldBackground(!isJSP11));
    _largeIconLabel.setEnabled(!isJSP11);
    _largeIconText.setEnabled(!isJSP11);
    _largeIconText.setBackground(getFieldBackground(!isJSP11));
  }

  /**
   * Check for user input error.
   *
   * The user has completed this page so check for input error.  If any, then
   * report the error in a dialog and throw an exception.
   *
   * @throws  VetoException  if a validation error arises
   */
  public void checkPage() throws VetoException {
    // Short name must be specified (unless no tags found)
    checkCondition(_shortName == null || getFileName().length() > 0,
      res.getString("FileNameMandatory"), _fileField);
    // Short name must be specified (unless no tags found)
    checkCondition(_shortName == null || getShortName().length() > 0,
      res.getString("ShortNameMandatory"), _shortNameText);
  }

  /**
   * Fetches file name for the taglib.
   *
   * @return  the taglib file name
   */
  public String getFileName() { return _fileField.getText(); }

  /**
   * Fetches short name for the taglib.
   *
   * @return  the taglib short name
   */
  public String getShortName() { return _shortNameText.getText(); }

  /**
   * Fetches URI for the taglib.
   *
   * @return  the taglib URI
   */
  public String getUri() { return _uriText.getText(); }

  /**
   * Fetches description for the taglib.
   *
   * @return  the taglib description
   */
  public String getDescription() { return _infoText.getText(); }

  /**
   * Fetches display name for the taglib.
   *
   * @return  the taglib display name
   */
  public String getDisplayName() {
    return (_jsp11Radio.isSelected() ? "" : _displayNameText.getText());
  }

  /**
   * Fetches filename for the small icon for the taglib.
   *
   * @return  the taglib small icon filename
   */
  public String getSmallIconName() {
    return (_jsp11Radio.isSelected() ? "" : _smallIconText.getText());
  }

  /**
   * Fetches filename for the large icon for the taglib.
   *
   * @return  the taglib large icon filename
   */
  public String getLargeIconName() {
    return (_jsp11Radio.isSelected() ? "" : _largeIconText.getText());
  }
}
