package wood.keith.opentools.wizards.jsptags;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.util.ArrayList;
import javax.servlet.jsp.tagext.TagLibraryValidator;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.borland.jbuilder.ide.PackageBrowserTree;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.util.VetoException;

/**
 * This class contains the wizard page which allows the user to specify
 * the details about JSP 1.2 validator extensions for the taglib as a whole.
 *
 * @author   Keith Wood
 * @version  2.0  6 February 2002 - JSP 1.2
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorWizardPage2 extends TaglibDescriptorWizardPage {

  // Internal variables  
  private TaglibDescriptorParameter _lastParam = null;
  private ArrayList _validators = null;

  // UI variables
  private ArrayList _listenerDescs = new ArrayList();
	private BorderLayout _borderLayout = new BorderLayout();
	private JPanel _bodyPanel = new JPanel();
	private GridBagLayout _gridBagLayout = new GridBagLayout();
	private JLabel _jLabel1 = new JLabel();
	private JComboBox _validatorCombo = new JComboBox();
  private JButton _validatorButton = new JButton();
	private JLabel _validatorLabel = new JLabel();
	private JScrollPane _validatorScroll = new JScrollPane();
	private JTextArea _validatorField = new JTextArea();
	private JLabel _paramLabel = new JLabel();
	private JButton _addParamButton = new JButton();
	private JButton _deleteParamButton = new JButton();
  private JLabel _paramValueLabel = new JLabel();
  private JTextField _paramValueField = new JTextField();
  private DefaultComboBoxModel _paramModel = new DefaultComboBoxModel();
	private JComboBox _paramCombo = new JComboBox(_paramModel);
	private Border _textBorder;
  private JScrollPane _paramDescScroll = new JScrollPane();
  private JTextArea _paramDescField = new JTextArea();
  private JLabel _paramDescLabel = new JLabel();
	private DocumentListener _docListener = new DocumentListener() {
		public void changedUpdate(DocumentEvent e) {
			updateControls();
		}
		public void insertUpdate(DocumentEvent e) {
			updateControls();
		}
		public void removeUpdate(DocumentEvent e) {
			updateControls();
		}
	};

  /**
   * Create a new wizard page.
   *
   * @param  wizard      the wizard invoking this page
   * @param  validators  a list of validators in this project
   */
  public TaglibDescriptorWizardPage2(TaglibDescriptorWizard wizard,
      ArrayList validators) {
    super(wizard, 2);
    _validators = validators;
    try {
      jbInit();
      initialize();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

	/**
	 * Define the user interface.
	 *
	 * The user interface is constructed here so the JBuilder UI Designer
	 * may be used to edit it.
   * 
   * @throws  Exception  if an error arises
	 */
	protected void jbInit() throws Exception {
		_textBorder = BorderFactory.createEtchedBorder(SystemColor.controlLtHighlight,SystemColor.controlShadow);
		this.setPageTitle(res.getString("TLDPage2Title"));
		try {
			this.setLargeIcon(_icon);
		}
		catch (Exception ex) {
			// Ignore
		}
		this.setInstructions(res.getString("TLDPage2Instructions"));
		this.setLayout(_borderLayout);

		_jLabel1.setDisplayedMnemonic(res.getString("FieldValidatorChar").charAt(0));
		_jLabel1.setLabelFor(_validatorCombo);
		_jLabel1.setText(res.getString("FieldValidatorName"));
		_validatorCombo.setMinimumSize(new Dimension(126, 21));
		_validatorCombo.setPreferredSize(new Dimension(130, 21));
		_validatorCombo.setToolTipText(res.getString("FieldValidatorTip"));
		_validatorCombo.setEditable(false);
		_validatorCombo.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				_validatorCombo_itemStateChanged(e);
			}
		});
    _validatorButton.setMaximumSize(new Dimension(51, 21));
    _validatorButton.setMinimumSize(new Dimension(51, 21));
    _validatorButton.setPreferredSize(new Dimension(51, 21));
    _validatorButton.setToolTipText(res.getString("ButtonValidatorTip"));
    _validatorButton.setMargin(new Insets(2, 4, 2, 4));
    _validatorButton.setMnemonic(res.getString("ButtonValidatorChar").charAt(0));
    _validatorButton.setText(res.getString("ButtonValidatorName"));
    _validatorButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _validatorButton_actionPerformed(e);
      }
    });
		_validatorLabel.setDisplayedMnemonic(res.getString("FieldValidatorDescriptionChar").charAt(0));
		_validatorLabel.setLabelFor(_validatorField);
		_validatorLabel.setText(res.getString("FieldValidatorDescriptionName"));
		_validatorScroll.setBorder(_textBorder);
		_validatorScroll.setPreferredSize(new Dimension(200, 40));
		_validatorField.setBorder(null);
		_validatorField.setToolTipText(res.getString("FieldValidatorDescriptionTip"));
		_validatorField.setFont(new java.awt.Font("SansSerif", 0, 12));
		_paramLabel.setDisplayedMnemonic(res.getString("FieldValidatorParameterChar").charAt(0));
		_paramLabel.setLabelFor(_paramCombo);
		_paramLabel.setText(res.getString("FieldValidatorParameterName"));
    _paramCombo.setMinimumSize(new Dimension(126, 21));
    _paramCombo.setPreferredSize(new Dimension(130, 21));
    _paramCombo.setToolTipText(res.getString("FieldValidatorParameterTip"));
    _paramCombo.setEditable(false);
    _paramCombo.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _paramCombo_itemStateChanged(e);
      }
    });
		_addParamButton.setMaximumSize(new Dimension(51, 21));
		_addParamButton.setMinimumSize(new Dimension(51, 21));
		_addParamButton.setPreferredSize(new Dimension(51, 21));
		_addParamButton.setToolTipText(res.getString("ButtonAddValidatorParameterTip"));
		_addParamButton.setMargin(new Insets(2, 4, 2, 4));
		_addParamButton.setMnemonic(res.getString("ButtonAddValidatorParameterChar").charAt(0));
		_addParamButton.setText(res.getString("ButtonAddValidatorParameterName"));
		_addParamButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_addParamButton_actionPerformed(e);
			}
		});
		_deleteParamButton.setMaximumSize(new Dimension(51, 21));
		_deleteParamButton.setMinimumSize(new Dimension(51, 21));
		_deleteParamButton.setPreferredSize(new Dimension(51, 21));
		_deleteParamButton.setToolTipText(res.getString("ButtonDeleteValidatorParameterTip"));
		_deleteParamButton.setMargin(new Insets(2, 4, 2, 4));
		_deleteParamButton.setMnemonic(res.getString("ButtonDeleteValidatorParameterChar").charAt(0));
		_deleteParamButton.setText(res.getString("ButtonDeleteValidatorParameterName"));
		_deleteParamButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_deleteParamButton_actionPerformed(e);
			}
		});
    _paramValueLabel.setDisplayedMnemonic(res.getString("FieldValidatorParameterValueChar").charAt(0));
    _paramValueLabel.setLabelFor(_paramValueField);
    _paramValueLabel.setText(res.getString("FieldValidatorParameterValueName"));
    _paramValueField.setToolTipText(res.getString("FieldValidatorParameterValueTip"));
    _paramValueField.getDocument().addDocumentListener(_docListener);
    _paramDescLabel.setDisplayedMnemonic(res.getString("FieldValidatorParameterDescriptionChar").charAt(0));
    _paramDescLabel.setLabelFor(_paramDescField);
    _paramDescLabel.setText(res.getString("FieldValidatorParameterDescriptionName"));
    _paramDescField.setText("");
    _paramDescField.setToolTipText(res.getString("FieldValidatorParameterDescriptionTip"));
    this.add(_bodyPanel, BorderLayout.CENTER);
		_bodyPanel.setLayout(_gridBagLayout);

		_bodyPanel.add(_jLabel1,        new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_validatorCombo,        new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _bodyPanel.add(_validatorButton,        new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_validatorLabel,        new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_validatorScroll,        new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 0), 0, 0));
    _validatorScroll.getViewport().add(_validatorField, null);
		_bodyPanel.add(_paramLabel,        new GridBagConstraints(0, 2, 1, 3, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_paramCombo,         new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
		_bodyPanel.add(_addParamButton,        new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_paramValueLabel,        new GridBagConstraints(0, 3, 1, 3, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_paramValueField,      new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
		_bodyPanel.add(_deleteParamButton,        new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_paramDescLabel,    new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    _bodyPanel.add(_paramDescScroll,       new GridBagConstraints(1, 4, 1, 1, 0.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 2), 0, 0));
    _paramDescScroll.getViewport().add(_paramDescField, null);
	}

	/**
	 * Initialise UI.
	 */
	private void initialize() {
		_validatorCombo.addItem(NO_ENTRY);
		if (_validators != null) {
			for (int index = 0; index < _validators.size(); index++) {
				_validatorCombo.addItem(_validators.get(index));
			}
		}
		_validatorCombo.setSelectedIndex(_validators.size());
		updateControls();
	}

  /**
   * Synchronise controls with field contents and JSP version.
	 */
	protected void updateControls() {
    Object obj = _validatorCombo.getSelectedItem();
    boolean enabled = 
        (!_validatorCombo.getSelectedItem().equals(NO_ENTRY) &&
        (obj instanceof TaglibDescriptorValidator || 
        ((String)obj).length() > 0));
    _validatorLabel.setEnabled(enabled);
    _validatorField.setEnabled(enabled);
    _validatorField.setBackground(getFieldBackground(enabled));
    _paramLabel.setEnabled(enabled);
    _paramCombo.setEnabled(enabled);
    _addParamButton.setEnabled(enabled);
    boolean paramEnabled = enabled && (_paramCombo.getSelectedIndex() > -1);
		_deleteParamButton.setEnabled(paramEnabled);
    _paramValueLabel.setEnabled(paramEnabled);
    _paramValueField.setEnabled(paramEnabled);
    _paramValueField.setBackground(getFieldBackground(paramEnabled));
    _paramDescLabel.setEnabled(paramEnabled);
    _paramDescField.setEnabled(paramEnabled);
    _paramDescField.setBackground(getFieldBackground(paramEnabled));
	}

	/**
	 * Check for user input error.
	 *
	 * The user has completed this page so check for input error.  If any, then
	 * report the error in a dialog and throw an exception.
   * 
   * @throws  VetoException  if not valid
	 */
	public void checkPage() throws VetoException {
    // Ensure any changes to the description are saved
    _paramCombo_itemStateChanged(null);
	}

	/**
	 * Fetches the validator class for the taglib.
	 *
	 * @return  the validator class name
	 */
	public TaglibDescriptorValidator getValidator() {
    TaglibDescriptorValidator validator = null;
    Object selection = _validatorCombo.getSelectedItem();
		String className = (selection.equals(NO_ENTRY) ? "" : selection.toString());
    if (className.length() > 0) {
      validator = 
          new TaglibDescriptorValidator(className, _validatorField.getText());
      for (int index = 0; index < _paramModel.getSize(); index++) {
        validator.getParameters().add(_paramModel.getElementAt(index));
      }
    }
    return validator;
	}

	/**
	 * Dis/enable validator fields based on entry of class name.
   * 
   * @param  e  the triggering event
	 */
	void _validatorCombo_itemStateChanged(ItemEvent e) {
		Object obj = _validatorCombo.getSelectedItem();
		_paramModel.removeAllElements();
		if (obj instanceof TaglibDescriptorValidator) {
			TaglibDescriptorValidator validator = (TaglibDescriptorValidator)obj;
			_validatorField.setText(validator.getDescription());
			for (int index = 0; index < validator.getParameters().size(); index++) {
				_paramModel.addElement(validator.getParameters().get(index));
			}
		}
		else {
			_validatorField.setText("");
		}
    updateControls();
	}

  /**
   * Select the validator class.
   * 
   * @param  e  the triggering event
   */
  void _validatorButton_actionPerformed(ActionEvent e) {
    String className = PackageBrowserTree.browseClass(
        (JBProject)wizardHost.getBrowser().getActiveProject(), this, 
        res.getString("ChooseValidatorTitle"), 
        _validatorCombo.getSelectedItem().toString());
    if (className != null && className.length() > 0) {
      Class validatorClass = null;
      try {
        validatorClass = _wizard.getClassLoader().loadClass(className);
      }
      catch (ClassNotFoundException ex) {
        JOptionPane.showMessageDialog(this, res.getString("NoClassFoundError"),
            getPageTitle(), JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (!TagLibraryValidator.class.isAssignableFrom(validatorClass)) {
        JOptionPane.showMessageDialog(this, 
            res.getString("NotATagLibraryValidatorError"), 
            getPageTitle(), JOptionPane.WARNING_MESSAGE);
      }
      _validatorCombo.setSelectedItem(className);
      _validatorField.requestFocus();
      updateControls();
    }
  }

	/**
	 * Update buttons based on table selection.
   * 
   * @param  e  the triggering event
	 */
	void _paramCombo_itemStateChanged(ItemEvent e) {
    if (_lastParam != null) {
      _lastParam.setValue(_paramValueField.getText());
      _lastParam.setDescription(_paramDescField.getText());
    }
		_lastParam = (TaglibDescriptorParameter)_paramCombo.getSelectedItem();
    _paramValueField.setText(_lastParam == null ? "" : _lastParam.getValue());
    _paramDescField.setText(
        _lastParam == null ? "" : _lastParam.getDescription());
		updateControls();
	}

	/**
	 * Add a new parameter to the list.
   * 
   * @param  e  the triggering event
	 */
	void _addParamButton_actionPerformed(ActionEvent e) {
    String name = JOptionPane.showInputDialog(
        this, res.getString("EnterParameterNamePrompt"), 
        getPageTitle(), JOptionPane.QUESTION_MESSAGE);
    if (name != null) {
      for (int index = 0; index < _paramModel.getSize(); index++) {
        TaglibDescriptorParameter param = 
            (TaglibDescriptorParameter)_paramModel.getElementAt(index);
        if (name.equals(param.getDisplayName())) {
          _paramCombo.setSelectedIndex(index);
          updateControls();
          return;
        }
      }
      _paramModel.addElement(new TaglibDescriptorParameter(name, "", ""));
      _paramCombo.setSelectedIndex(_paramModel.getSize() - 1);
      _paramValueField.requestFocus();
      updateControls();
    }
	}

	/**
	 * Delete a parameter from the list.
   * 
   * @param  e  the triggering event
	 */
	void _deleteParamButton_actionPerformed(ActionEvent e) {
    _lastParam = null;
		_paramModel.removeElementAt(_paramCombo.getSelectedIndex());
		updateControls();
	}
}
