package wood.keith.opentools.wizards.jsptags;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.util.ArrayList;
import javax.servlet.ServletContextAttributeListener;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletRequestAttributeListener;
import javax.servlet.ServletRequestListener;
import javax.servlet.http.HttpSessionActivationListener;
import javax.servlet.http.HttpSessionAttributeListener;
import javax.servlet.http.HttpSessionBindingListener;
import javax.servlet.http.HttpSessionListener;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.borland.jbuilder.ide.PackageBrowserTree;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.util.VetoException;

/**
 * This class contains the wizard page which allows the user to specify
 * the details about JSP 1.2 listener extensions for the taglib as a whole.
 *
 * @author   Keith Wood
 * @version  2.0  6 February 2002 - JSP 1.2
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorWizardPage3 extends TaglibDescriptorWizardPage {

  // Internal variables  
  private TaglibDescriptorListener _lastListener = null;

  // UI variables
	private BorderLayout _borderLayout = new BorderLayout();
	private JPanel _bodyPanel = new JPanel();
	private GridBagLayout _gridBagLayout = new GridBagLayout();
	private JLabel _descrLabel = new JLabel();
	private JScrollPane _descrScroll = new JScrollPane();
	private JTextArea _descrField = new JTextArea();
	private JLabel _jLabel3 = new JLabel();
  private DefaultComboBoxModel _listenerModel = new DefaultComboBoxModel();
  private JComboBox _listenerCombo = new JComboBox(_listenerModel);
	private JButton _addListenerButton = new JButton();
	private JButton _deleteListenerButton = new JButton();
  private JLabel _displayLabel = new JLabel();
  private JTextField _displayField = new JTextField();
  private JLabel _smallIconLabel = new JLabel();
  private JTextField _smallIconField = new JTextField();
  private JLabel _largeIconLabel = new JLabel();
  private JTextField _largeIconField = new JTextField();
	private Border _textBorder;
	private DocumentListener _docListener = new DocumentListener() {
		public void changedUpdate(DocumentEvent e) {
			updateControls();
		}
		public void insertUpdate(DocumentEvent e) {
			updateControls();
		}
		public void removeUpdate(DocumentEvent e) {
			updateControls();
		}
	};

  /**
   * Create a new wizard page.
   *
   * @param  wizard     the wizard invoking this page
   * @param  listeners  a list of listeners in this project
   */
  public TaglibDescriptorWizardPage3(TaglibDescriptorWizard wizard,
      ArrayList listeners) {
    super(wizard, 3);
    try {
      jbInit();
      initialize(listeners);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

	/**
	 * Define the user interface.
	 *
	 * The user interface is constructed here so the JBuilder UI Designer
	 * may be used to edit it.
   * 
   * @throws  Exception  if an error arises
	 */
	protected void jbInit() throws Exception {
		_textBorder = BorderFactory.createEtchedBorder(SystemColor.controlLtHighlight,SystemColor.controlShadow);
		this.setPageTitle(res.getString("TLDPage3Title"));
		try {
			this.setLargeIcon(_icon);
		}
		catch (Exception ex) {
			// Ignore
		}
		this.setInstructions(res.getString("TLDPage3Instructions"));
		this.setLayout(_borderLayout);

    _jLabel3.setDisplayedMnemonic(res.getString("FieldListenersChar").charAt(0));
    _jLabel3.setText(res.getString("FieldListenersName"));
    _jLabel3.setLabelFor(_listenerCombo);
    _listenerCombo.setMinimumSize(new Dimension(126, 21));
    _listenerCombo.setPreferredSize(new Dimension(130, 21));
    _listenerCombo.setToolTipText(res.getString("FieldListenersTip"));
    _listenerCombo.setEditable(false);
    _listenerCombo.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _listenerCombo_itemStateChanged(e);
      }
    });
		_descrLabel.setDisplayedMnemonic(res.getString("FieldListenerDescriptionChar").charAt(0));
		_descrLabel.setLabelFor(_descrField);
		_descrLabel.setText(res.getString("FieldListenerDescriptionName"));
		_descrScroll.setBorder(_textBorder);
		_descrScroll.setPreferredSize(new Dimension(200, 40));
		_descrField.setBorder(null);
		_descrField.setToolTipText(res.getString("FieldListenerDescriptionTip"));
		_descrField.setFont(new java.awt.Font("SansSerif", 0, 12));
		_addListenerButton.setMaximumSize(new Dimension(81, 21));
		_addListenerButton.setMinimumSize(new Dimension(37, 21));
		_addListenerButton.setPreferredSize(new Dimension(37, 21));
		_addListenerButton.setToolTipText(res.getString("ButtonAddListenerTip"));
		_addListenerButton.setMargin(new Insets(2, 4, 2, 4));
		_addListenerButton.setMnemonic(res.getString("ButtonAddListenerChar").charAt(0));
		_addListenerButton.setText(res.getString("ButtonAddListenerName"));
		_addListenerButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_addListenerButton_actionPerformed(e);
			}
		});
		_deleteListenerButton.setMaximumSize(new Dimension(51, 21));
		_deleteListenerButton.setMinimumSize(new Dimension(51, 21));
		_deleteListenerButton.setPreferredSize(new Dimension(51, 21));
		_deleteListenerButton.setToolTipText(res.getString("ButtonDeleteListenerTip"));
		_deleteListenerButton.setMargin(new Insets(2, 4, 2, 4));
		_deleteListenerButton.setMnemonic(res.getString("ButtonDeleteListenerChar").charAt(0));
		_deleteListenerButton.setText(res.getString("ButtonDeleteListenerName"));
		_deleteListenerButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_deleteListenerButton_actionPerformed(e);
			}
		});
    _displayLabel.setDisplayedMnemonic(res.getString("FieldListenerDisplayNameChar").charAt(0));
    _displayLabel.setLabelFor(_displayField);
    _displayLabel.setText(res.getString("FieldListenerDisplayNameName"));
    _displayField.setToolTipText(res.getString("FieldListenerDisplayNameTip"));
    _smallIconLabel.setDisplayedMnemonic(res.getString("FieldListenerSmallIconChar").charAt(0));
    _smallIconLabel.setLabelFor(_smallIconField);
    _smallIconLabel.setText(res.getString("FieldListenerSmallIconName"));
    _smallIconField.setToolTipText(res.getString("FieldListenerSmallIconTip"));
    _largeIconLabel.setDisplayedMnemonic(res.getString("FieldListenerLargeIconChar").charAt(0));
    _largeIconLabel.setLabelFor(_largeIconField);
    _largeIconLabel.setText(res.getString("FieldListenerLargeIconName"));
    _largeIconField.setToolTipText(res.getString("FieldListenerLargeIconTip"));
    this.add(_bodyPanel, BorderLayout.CENTER);
		_bodyPanel.setLayout(_gridBagLayout);

		_bodyPanel.add(_jLabel3,        new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_listenerCombo,  new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 4, 2, 2), 0, 0));
		_bodyPanel.add(_addListenerButton,        new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_displayLabel,  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_displayField,   new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_deleteListenerButton,         new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_smallIconLabel,  new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_smallIconField,   new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_largeIconLabel,  new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_largeIconField,   new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_descrLabel,        new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_descrScroll,         new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 2), 0, 0));
		_descrScroll.getViewport().add(_descrField, null);
	}

	/**
	 * Initialise UI.
   * 
   * @param  listeners  a list of listeners in this project
	 */
	private void initialize(ArrayList listeners) {
		if (listeners != null) {
			for (int index = 0; index < listeners.size(); index++) {
				_listenerModel.addElement(listeners.get(index));
			}
		}
		updateControls();
	}

  /**
   * Synchronise controls with field contents and JSP version.
   */
  protected void updateControls() {
		boolean selected = (_listenerCombo.getSelectedIndex() > -1);
    boolean isJSP20 = _jspVersion.equals(JSPTagProperties.JSP_20);
    _deleteListenerButton.setEnabled(selected);
    _displayLabel.setEnabled(selected && isJSP20);
    _displayField.setEnabled(selected && isJSP20);
    _displayField.setBackground(getFieldBackground(selected && isJSP20));
    _smallIconLabel.setEnabled(selected && isJSP20);
    _smallIconField.setEnabled(selected && isJSP20);
    _smallIconField.setBackground(getFieldBackground(selected && isJSP20));
    _largeIconLabel.setEnabled(selected && isJSP20);
    _largeIconField.setEnabled(selected && isJSP20);
    _largeIconField.setBackground(getFieldBackground(selected && isJSP20));
    _descrLabel.setEnabled(selected && isJSP20);
    _descrField.setEnabled(selected && isJSP20);
    _descrField.setBackground(getFieldBackground(selected && isJSP20));
	}

	/**
	 * Check for user input error.
	 *
	 * The user has completed this page so check for input error.  If any, then
	 * report the error in a dialog and throw an exception.
   * 
   * @throws  VetoException  if not valid
	 */
	public void checkPage() throws VetoException {
    // Ensure any changes to the description are saved
    _listenerCombo_itemStateChanged(null);
	}

	/**
	 * Update buttons based on listener selection.
   * 
   * @param  e  the triggering event
	 */
	void _listenerCombo_itemStateChanged(ItemEvent e) {
    if (_lastListener != null) {
      _lastListener.setDisplayName(_displayField.getText());
      _lastListener.setSmallIcon(_smallIconField.getText());
      _lastListener.setLargeIcon(_largeIconField.getText());
      _lastListener.setDescription(_descrField.getText());
    }
    _lastListener = (TaglibDescriptorListener)_listenerCombo.getSelectedItem();
    _displayField.setText(
        _lastListener == null ? "" : _lastListener.getDisplayName());
    _smallIconField.setText(
        _lastListener == null ? "" : _lastListener.getSmallIcon());
    _largeIconField.setText(
        _lastListener == null ? "" : _lastListener.getLargeIcon());
    _descrField.setText(
        _lastListener == null ? "" : _lastListener.getDescription());
		updateControls();
	}

	/**
	 * Add a new listener to the list.
   * 
   * @param  e  the triggering event
	 */
	void _addListenerButton_actionPerformed(ActionEvent e) {
    String className = PackageBrowserTree.browseClass(
        (JBProject)wizardHost.getBrowser().getActiveProject(), this, 
        res.getString("ChooseListenerTitle"), "");
    if (className != null && className.length() > 0) {
      Class listenerClass = null;
      try {
        listenerClass = _wizard.getClassLoader().loadClass(className);
      }
      catch (ClassNotFoundException ex) {
        JOptionPane.showMessageDialog(this, res.getString("NoClassFoundError"),
            getPageTitle(), JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (!(HttpSessionActivationListener.class.isAssignableFrom(listenerClass) ||
          HttpSessionAttributeListener.class.isAssignableFrom(listenerClass) ||
          HttpSessionBindingListener.class.isAssignableFrom(listenerClass) ||
          HttpSessionListener.class.isAssignableFrom(listenerClass) ||
          ServletContextAttributeListener.class.isAssignableFrom(listenerClass) ||
          ServletContextListener.class.isAssignableFrom(listenerClass) ||
          ServletRequestAttributeListener.class.isAssignableFrom(listenerClass) ||
          ServletRequestListener.class.isAssignableFrom(listenerClass))) {
        JOptionPane.showMessageDialog(this, 
            res.getString("NotAServletListenerError"), 
            getPageTitle(), JOptionPane.WARNING_MESSAGE);
      }
      for (int index = 0; index < _listenerModel.getSize(); index++) {
        TaglibDescriptorListener listener = 
            (TaglibDescriptorListener)_listenerModel.getElementAt(index);
        if (className.equals(listener.getClassName())) {
          _listenerCombo.setSelectedIndex(index);
          updateControls();
          return;
        }
      }
      _listenerModel.addElement(new TaglibDescriptorListener(className, ""));
      _listenerCombo.setSelectedIndex(_listenerModel.getSize() - 1);
      _displayField.requestFocus();
      updateControls();
    }
	}

	/**
	 * Delete a listener from the list.
   * 
   * @param  e  the triggering event
	 */
	void _deleteListenerButton_actionPerformed(ActionEvent e) {
    _lastListener = null;
    _listenerModel.removeElementAt(_listenerCombo.getSelectedIndex());
		_listenerCombo.requestFocus();
		updateControls();
	}

  /**
   * Fetches the list of taglib listeners.
   *
   * @return  the taglib listeners as name/description pairs
   */
  public ArrayList getListeners() {
    ArrayList listeners = new ArrayList();
    for (int index = 0; index < _listenerModel.getSize(); index++) {
      listeners.add(_listenerModel.getElementAt(index));
    }
    return listeners;
  }
}
