package wood.keith.opentools.wizards.jsptags;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.io.File;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.filechooser.FileFilter;

import com.borland.primetime.util.VetoException;

/**
 * This class contains the wizard page which allows the user to specify
 * the details about JSP 2.0 tag file extensions for the taglib as a whole.
 *
 * @author   Keith Wood
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorWizardPage4 extends TaglibDescriptorWizardPage {
  
  private static final String TAG_FILE_EXT = ".tag";
  private static final String TAGX_FILE_EXT = ".tagx";
  private static final String TAG_FILE_DESC = res.getString("TagFilesName");
  
  // Internal variables
  private TaglibDescriptorTagFile _lastFile = null;

  // UI variables
  private ArrayList _listenerDescs = new ArrayList();
	private BorderLayout _borderLayout = new BorderLayout();
	private JPanel _bodyPanel = new JPanel();
	private GridBagLayout _gridBagLayout = new GridBagLayout();
	private JLabel _fileLabel = new JLabel();
  private DefaultComboBoxModel _fileModel = new DefaultComboBoxModel();
	private JComboBox _fileCombo = new JComboBox(_fileModel);
  private JButton _addFileButton = new JButton();
	private JLabel _displayLabel = new JLabel();
  private JTextField _displayField = new JTextField();
  private JButton _deleteFileButton = new JButton();
	private JLabel _smallIconLabel = new JLabel();
	private JTextField _smallIconField = new JTextField();
  private JLabel _largeIconLabel = new JLabel();
  private JTextField _largeIconField = new JTextField();
  private JLabel _pathLabel = new JLabel();
  private JTextField _pathField = new JTextField();
  private JButton _pathButton = new JButton();
  private JLabel _descriptionLabel = new JLabel();
  private JScrollPane _descriptionScroll = new JScrollPane();
  private JTextArea _descriptionField = new JTextArea();
  private JLabel _exampleLabel = new JLabel();
  private JScrollPane _exampleScroll = new JScrollPane();
  private JTextArea _exampleField = new JTextArea();
	private Border _textBorder;

  /**
   * Create a new wizard page.
   *
   * @param  wizard    the wizard invoking this page
   * @param  tagFiles  the list of tag files found
   */
  public TaglibDescriptorWizardPage4(TaglibDescriptorWizard wizard, 
      ArrayList tagFiles) {
    super(wizard, 4);
    try {
      jbInit();
      initialize(tagFiles);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

	/**
	 * Define the user interface.
	 *
	 * The user interface is constructed here so the JBuilder UI Designer
	 * may be used to edit it.
   * 
   * @throws  Exception  if an error arises
	 */
	protected void jbInit() throws Exception {
		_textBorder = BorderFactory.createEtchedBorder(SystemColor.controlLtHighlight,SystemColor.controlShadow);
		this.setPageTitle(res.getString("TLDPage4Title"));
		try {
			this.setLargeIcon(_icon);
		}
		catch (Exception ex) {
			// Ignore
		}
		this.setInstructions(res.getString("TLDPage4Instructions"));
		this.setLayout(_borderLayout);

		_fileLabel.setDisplayedMnemonic(res.getString("FieldFileChar").charAt(0));
		_fileLabel.setLabelFor(_fileCombo);
		_fileLabel.setText(res.getString("FieldFileName"));
		_fileCombo.setMinimumSize(new Dimension(126, 21));
		_fileCombo.setPreferredSize(new Dimension(130, 21));
		_fileCombo.setToolTipText(res.getString("FieldFileTip"));
		_fileCombo.setEditable(false);
		_fileCombo.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				_fileCombo_itemStateChanged(e);
			}
		});
		_displayLabel.setDisplayedMnemonic(res.getString("FieldTagFileDisplayNameChar").charAt(0));
		_displayLabel.setLabelFor(_displayField);
		_displayLabel.setText(res.getString("FieldTagFileDisplayNameName"));
		_displayField.setToolTipText(res.getString("FieldTagFileDisplayNameTip"));
		_addFileButton.setMaximumSize(new Dimension(51, 21));
		_addFileButton.setMinimumSize(new Dimension(51, 21));
		_addFileButton.setPreferredSize(new Dimension(51, 21));
		_addFileButton.setToolTipText(res.getString("ButtonAddTagFileTip"));
		_addFileButton.setMargin(new Insets(2, 4, 2, 4));
		_addFileButton.setMnemonic(res.getString("ButtonAddTagFileChar").charAt(0));
		_addFileButton.setText(res.getString("ButtonAddTagFileName"));
		_addFileButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_addFileButton_actionPerformed(e);
			}
		});
		_deleteFileButton.setMaximumSize(new Dimension(51, 21));
		_deleteFileButton.setMinimumSize(new Dimension(51, 21));
		_deleteFileButton.setPreferredSize(new Dimension(51, 21));
		_deleteFileButton.setToolTipText(res.getString("ButtonDeleteTagFileTip"));
		_deleteFileButton.setMargin(new Insets(2, 4, 2, 4));
		_deleteFileButton.setMnemonic(res.getString("ButtonDeleteTagFileChar").charAt(0));
		_deleteFileButton.setText(res.getString("ButtonDeleteTagFileName"));
		_deleteFileButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_deleteFileButton_actionPerformed(e);
			}
		});
    _smallIconLabel.setDisplayedMnemonic(res.getString("FieldTagFileSmallIconChar").charAt(0));
    _smallIconLabel.setLabelFor(_smallIconField);
    _smallIconLabel.setText(res.getString("FieldTagFileSmallIconName"));
    _smallIconField.setToolTipText(res.getString("FieldTagFileSmallIconTip"));
    _largeIconLabel.setDisplayedMnemonic(res.getString("FieldTagFileLargeIconChar").charAt(0));
    _largeIconLabel.setLabelFor(_largeIconField);
    _largeIconLabel.setText(res.getString("FieldTagFileLargeIconName"));
    _largeIconField.setToolTipText(res.getString("FieldTagFileLargeIconTip"));
    _pathLabel.setDisplayedMnemonic(res.getString("FieldTagFilePathChar").charAt(0));
    _pathLabel.setLabelFor(_pathField);
    _pathLabel.setText(res.getString("FieldTagFilePathName"));
    _pathField.setToolTipText(res.getString("FieldTagFilePathTip"));
    _pathButton.setMaximumSize(new Dimension(51, 21));
    _pathButton.setMinimumSize(new Dimension(51, 21));
    _pathButton.setPreferredSize(new Dimension(51, 21));
    _pathButton.setToolTipText(res.getString("ButtonTagFilePathTip"));
    _pathButton.setMargin(new Insets(2, 4, 2, 4));
    _pathButton.setMnemonic(res.getString("ButtonTagFilePathChar").charAt(0));
    _pathButton.setText(res.getString("ButtonTagFilePathName"));
    _pathButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _pathButton_actionPerformed(e);
      }
    });
    _descriptionLabel.setDisplayedMnemonic(res.getString("FieldTagFileDescriptionChar").charAt(0));
    _descriptionLabel.setLabelFor(_descriptionField);
    _descriptionLabel.setText(res.getString("FieldTagFileDescriptionName"));
    _descriptionScroll.setBorder(_textBorder);
    _descriptionScroll.setPreferredSize(new Dimension(200, 40));
    _descriptionField.setBorder(null);
    _descriptionField.setToolTipText(res.getString("FieldTagFileDescriptionTip"));
    _descriptionField.setFont(new java.awt.Font("SansSerif", 0, 12));
    _exampleLabel.setDisplayedMnemonic(res.getString("FieldTagFileExampleChar").charAt(0));
    _exampleLabel.setLabelFor(_exampleField);
    _exampleLabel.setText(res.getString("FieldTagFileExampleName"));
    _exampleScroll.setBorder(_textBorder);
    _exampleScroll.setPreferredSize(new Dimension(200, 40));
    _exampleField.setBorder(null);
    _exampleField.setToolTipText(res.getString("FieldTagFileExampleTip"));
    _exampleField.setFont(new java.awt.Font("SansSerif", 0, 12));
    this.add(_bodyPanel, BorderLayout.CENTER);
		_bodyPanel.setLayout(_gridBagLayout);

		_bodyPanel.add(_fileLabel,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_fileCombo,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_addFileButton,  new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_displayLabel,  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_displayField,   new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_deleteFileButton,  new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_smallIconLabel,  new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_smallIconField,   new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_largeIconLabel,  new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_largeIconField,   new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_pathLabel,  new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_pathField,   new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_pathButton,  new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_descriptionLabel,  new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_descriptionScroll,    new GridBagConstraints(1, 5, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_exampleLabel,  new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
		_bodyPanel.add(_exampleScroll,    new GridBagConstraints(1, 6, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 2), 0, 0));
		_descriptionScroll.getViewport().add(_descriptionField, null);
		_exampleScroll.getViewport().add(_exampleField, null);
	}

	/**
	 * Initialise UI.
   *    
   * @param  tagFiles  the list of tag files found
	 */
	private void initialize(ArrayList tagFiles) {
    for (int index = 0; index < tagFiles.size(); index++) {
      _fileModel.addElement(tagFiles.get(index));
    }
		updateControls();
	}

  /**
   * Synchronise controls with field contents and JSP version.
   */
  protected void updateControls() {
    boolean enabled = (_fileModel.getSize() > 0);
    _deleteFileButton.setEnabled(enabled);
    _displayLabel.setEnabled(enabled);
    _displayField.setEnabled(enabled);
    _displayField.setBackground(getFieldBackground(enabled));
    _smallIconLabel.setEnabled(enabled);
    _smallIconField.setEnabled(enabled);
    _smallIconField.setBackground(getFieldBackground(enabled));
    _largeIconLabel.setEnabled(enabled);
    _largeIconField.setEnabled(enabled);
    _largeIconField.setBackground(getFieldBackground(enabled));
    _pathLabel.setEnabled(enabled);
    _pathField.setEnabled(enabled);
    _pathField.setBackground(getFieldBackground(enabled));
    _pathButton.setEnabled(enabled);
    _descriptionLabel.setEnabled(enabled);
    _descriptionField.setEnabled(enabled);
    _descriptionField.setBackground(getFieldBackground(enabled));
    _exampleLabel.setEnabled(enabled);
    _exampleField.setEnabled(enabled);
    _exampleField.setBackground(getFieldBackground(enabled));
	}

	/**
	 * Check for user input error.
	 *
	 * The user has completed this page so check for input error.  If any, then
	 * report the error in a dialog and throw an exception.
   * 
   * @throws  VetoException  if not valid
	 */
	public void checkPage() throws VetoException {
    // Ensure any changes to the fields are saved
    _fileCombo_itemStateChanged(null);
    int index = -1;
    try {
      for (index = 0; index < _fileModel.getSize(); index++) {
        String filename =
            ((TaglibDescriptorTagFile)_fileModel.getElementAt(index)).
            getPath().replace('\\', '/').toUpperCase();
        checkCondition(filename.length() > 0,
            res.getString("TagFilePathMandatory"), _pathField);
        checkCondition(filename.startsWith("/META-INF/TAGS/") ||
            filename.startsWith("/WEB-INF/TAGS/"),
            res.getString("TagFilePathRequiredFormat"), _pathField);
      }
    }
    catch (VetoException ve) {
      _fileCombo.setSelectedIndex(index);
      throw ve;
    }
	}

  /**
   * Populate fields from selected tag file.
   * 
   * @param  e  the triggering event
   */
  void _fileCombo_itemStateChanged(ItemEvent e) {
    if (_lastFile != null) {
      _lastFile.setDisplayName(_displayField.getText());
      _lastFile.setSmallIcon(_smallIconField.getText());
      _lastFile.setLargeIcon(_largeIconField.getText());
      _lastFile.setPath(_pathField.getText());
      _lastFile.setDescription(_descriptionField.getText());
      _lastFile.setExample(_exampleField.getText());
    }
    _lastFile = (TaglibDescriptorTagFile)_fileCombo.getSelectedItem();
    _displayField.setText(_lastFile == null ? "" : _lastFile.getDisplayName());
    _smallIconField.setText(_lastFile == null ? "" : _lastFile.getSmallIcon());
    _largeIconField.setText(_lastFile == null ? "" : _lastFile.getLargeIcon());
    _pathField.setText(_lastFile == null ? "" : _lastFile.getPath());
    _descriptionField.setText(
        _lastFile == null ? "" : _lastFile.getDescription());
    _exampleField.setText(_lastFile == null ? "" : _lastFile.getExample());
  }

  /**
   * Add a new tag file to the list.
   * 
   * @param  e  the triggering event
   */
  void _addFileButton_actionPerformed(ActionEvent e) {
    String name = JOptionPane.showInputDialog(
        this, res.getString("TagFileShortNamePrompt"), 
        getPageTitle(), JOptionPane.QUESTION_MESSAGE);
    if (name != null) {
      for (int index = 0; index < _fileModel.getSize(); index++) {
        TaglibDescriptorTagFile file = 
            (TaglibDescriptorTagFile)_fileModel.getElementAt(index);
        if (name.equals(file.getName())) {
          _fileCombo.setSelectedIndex(index);
          updateControls();
          return;
        }
      }
      _fileModel.addElement(
          new TaglibDescriptorTagFile(name, "/WEB-INF/tags/"));
      _fileCombo.setSelectedIndex(_fileModel.getSize() - 1);
      _displayField.requestFocus();
      updateControls();
    }
  }

  /**
   * Delete a tag file from the list.
   * 
   * @param  e  the triggering event
   */
  void _deleteFileButton_actionPerformed(ActionEvent e) {
    _lastFile = null;
    _fileModel.removeElement(_fileCombo.getSelectedItem());
    _fileCombo.setSelectedIndex(_fileModel.getSize() - 1);
    _displayField.requestFocus();
    updateControls();
  }

  /**
   * Select a tag file.
   * 
   * @param  e  the triggering event
   */
  void _pathButton_actionPerformed(ActionEvent e) {
    String projPath = 
        _wizard.getProject().getUrl().getFileObject().getAbsolutePath() + 
        File.separator + "WEB-INF" + File.separator + "tags" + File.separator;
    File file = new File(projPath);
    JFileChooser chooser = new JFileChooser();
    chooser.setDialogTitle(getPageTitle());
    chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    // Set filters for all and .tag/.tagx files, with the latter as the default
    chooser.addChoosableFileFilter(chooser.getAcceptAllFileFilter());
    chooser.setFileFilter(new FileFilter() {
      public boolean accept(File file) {
        String path = file.getPath();
        return (file.isDirectory() ||
          ((path.length() > TAG_FILE_EXT.length()) &&
          (path.substring(path.length() - TAG_FILE_EXT.length()).
          toLowerCase().equals(TAG_FILE_EXT))) ||
          ((path.length() > TAGX_FILE_EXT.length()) &&
          (path.substring(path.length() - TAGX_FILE_EXT.length()).
          toLowerCase().equals(TAGX_FILE_EXT))));
      }
      public String getDescription() {
        return TAG_FILE_DESC;
      }
    });
    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      String filename = 
          chooser.getSelectedFile().getAbsolutePath().replace('\\', '/');
      int pos = filename.toUpperCase().indexOf("/META-INF/TAGS/");
      if (pos == -1) {
        pos = filename.toUpperCase().indexOf("/WEB-INF/TAGS/");
      }
      if (pos == -1) {
        JOptionPane.showMessageDialog(this, res.getString("TagFilePathError"), 
            getPageTitle(), JOptionPane.ERROR_MESSAGE);
        return;
      }
      _pathField.setText(filename.substring(pos));
    }
  }

	/**
	 * Fetches the set of tag files for the taglib.
	 *
	 * @return  the tag file details
	 */
	public ArrayList getTagFiles() {
    ArrayList tagFiles = new ArrayList();
    for (int index = 0; index < _fileModel.getSize(); index++) {
      tagFiles.add(_fileModel.getElementAt(index));
    }
		return tagFiles;
	}
}
