package wood.keith.opentools.wizards.jsptags;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;

import com.borland.jbuilder.ide.PackageBrowserTree;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.util.VetoException;

/**
 * This class contains the wizard page which allows the user to specify
 * the details about JSP 2.0 function extensions for the taglib as a whole.
 *
 * @author   Keith Wood
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorWizardPage5 extends TaglibDescriptorWizardPage {
  
  /** A comparator for method signatures: "returnType name(parameters)". */
  private Comparator SIG_COMPARATOR = new Comparator() {
    public int compare(Object obj1, Object obj2) {
      return getName((String)obj1).compareTo(getName((String)obj2));
    }

    public boolean equals(Object obj) {
      return false;
    }
    
    private String getName(String signature) {
      int start = signature.indexOf(" ");
      int end = signature.indexOf("(", start + 1);
      return signature.substring(start + 1, end);
    }
  };
  
  // Internal variables
  private TaglibDescriptorFunction _lastFunction = null;

  // UI variables
  private ArrayList _listenerDescs = new ArrayList();
  private BorderLayout _borderLayout = new BorderLayout();
  private JPanel _bodyPanel = new JPanel();
  private GridBagLayout _gridBagLayout = new GridBagLayout();
  private JLabel _functionLabel = new JLabel();
  private DefaultComboBoxModel _functionModel = new DefaultComboBoxModel();
  private JComboBox _functionCombo = new JComboBox(_functionModel);
  private JButton _addFunctionButton = new JButton();
  private JLabel _displayLabel = new JLabel();
  private JTextField _displayField = new JTextField();
  private JButton _deleteFunctionButton = new JButton();
  private JLabel _smallIconLabel = new JLabel();
  private JTextField _smallIconField = new JTextField();
  private JLabel _largeIconLabel = new JLabel();
  private JTextField _largeIconField = new JTextField();
  private JLabel _classLabel = new JLabel();
  private JTextField _classField = new JTextField();
  private JButton _classButton = new JButton();
  private JLabel _signatureLabel = new JLabel();
  private DefaultComboBoxModel _signatureModel = new DefaultComboBoxModel();
  private JComboBox _signatureCombo = new JComboBox(_signatureModel);
  private JLabel _descriptionLabel = new JLabel();
  private JScrollPane _descriptionScroll = new JScrollPane();
  private JTextArea _descriptionField = new JTextArea();
  private JLabel _exampleLabel = new JLabel();
  private JScrollPane _exampleScroll = new JScrollPane();
  private JTextArea _exampleField = new JTextArea();
  private Border _textBorder;

  /**
   * Create a new wizard page.
   *
   * @param  wizard  the wizard invoking this page
   */
  public TaglibDescriptorWizardPage5(TaglibDescriptorWizard wizard) {
    super(wizard, 5);
    try {
      jbInit();
      initialize();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

	/**
	 * Define the user interface.
	 *
	 * The user interface is constructed here so the JBuilder UI Designer
	 * may be used to edit it.
   * 
   * @throws  Exception  if an error arises
	 */
	protected void jbInit() throws Exception {
    _textBorder = BorderFactory.createEtchedBorder(SystemColor.controlLtHighlight,SystemColor.controlShadow);
    this.setPageTitle(res.getString("TLDPage5Title"));
    try {
      this.setLargeIcon(_icon);
    }
    catch (Exception ex) {
      // Ignore
    }
    this.setInstructions(res.getString("TLDPage5Instructions"));
    this.setLayout(_borderLayout);

    _functionLabel.setDisplayedMnemonic(res.getString("FieldFunctionChar").charAt(0));
    _functionLabel.setLabelFor(_functionCombo);
    _functionLabel.setText(res.getString("FieldFunctionName"));
    _functionCombo.setMinimumSize(new Dimension(126, 21));
    _functionCombo.setPreferredSize(new Dimension(130, 21));
    _functionCombo.setToolTipText(res.getString("FieldFunctionTip"));
    _functionCombo.setEditable(false);
    _functionCombo.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _functionCombo_itemStateChanged(e);
      }
    });
    _displayLabel.setDisplayedMnemonic(res.getString("FieldFunctionDisplayNameChar").charAt(0));
    _displayLabel.setLabelFor(_displayField);
    _displayLabel.setText(res.getString("FieldFunctionDisplayNameName"));
    _displayField.setToolTipText(res.getString("FieldFunctionDisplayNameTip"));
    _addFunctionButton.setMaximumSize(new Dimension(51, 21));
    _addFunctionButton.setMinimumSize(new Dimension(51, 21));
    _addFunctionButton.setPreferredSize(new Dimension(51, 21));
    _addFunctionButton.setToolTipText(res.getString("ButtonAddFunctionTip"));
    _addFunctionButton.setMargin(new Insets(2, 4, 2, 4));
    _addFunctionButton.setMnemonic(res.getString("ButtonAddFunctionChar").charAt(0));
    _addFunctionButton.setText(res.getString("ButtonAddFunctionName"));
    _addFunctionButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _addFunctionButton_actionPerformed(e);
      }
    });
    _deleteFunctionButton.setMaximumSize(new Dimension(51, 21));
    _deleteFunctionButton.setMinimumSize(new Dimension(51, 21));
    _deleteFunctionButton.setPreferredSize(new Dimension(51, 21));
    _deleteFunctionButton.setToolTipText(res.getString("ButtonDeleteFunctionTip"));
    _deleteFunctionButton.setMargin(new Insets(2, 4, 2, 4));
    _deleteFunctionButton.setMnemonic(res.getString("ButtonDeleteFunctionChar").charAt(0));
    _deleteFunctionButton.setText(res.getString("ButtonDeleteFunctionName"));
    _deleteFunctionButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _deleteFunctionButton_actionPerformed(e);
      }
    });
    _smallIconLabel.setDisplayedMnemonic(res.getString("FieldFunctionSmallIconChar").charAt(0));
    _smallIconLabel.setLabelFor(_smallIconField);
    _smallIconLabel.setText(res.getString("FieldFunctionSmallIconName"));
    _smallIconField.setToolTipText(res.getString("FieldFunctionSmallIconTip"));
    _largeIconLabel.setDisplayedMnemonic(res.getString("FieldFunctionLargeIconChar").charAt(0));
    _largeIconLabel.setLabelFor(_largeIconField);
    _largeIconLabel.setText(res.getString("FieldFunctionLargeIconName"));
    _largeIconField.setToolTipText(res.getString("FieldFunctionLargeIconTip"));
    _classLabel.setText(res.getString("FieldFunctionClassName"));
    _classField.setToolTipText(res.getString("FieldFunctionClassTip"));
    _classField.setEditable(false);
    _classField.setBackground(getFieldBackground(false));
    _classButton.setMaximumSize(new Dimension(51, 21));
    _classButton.setMinimumSize(new Dimension(51, 21));
    _classButton.setPreferredSize(new Dimension(51, 21));
    _classButton.setToolTipText(res.getString("ButtonFunctionClassTip"));
    _classButton.setMargin(new Insets(2, 4, 2, 4));
    _classButton.setMnemonic(res.getString("ButtonFunctionClassChar").charAt(0));
    _classButton.setText(res.getString("ButtonFunctionClassName"));
    _classButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _classButton_actionPerformed(e);
      }
    });
    _signatureLabel.setDisplayedMnemonic(res.getString("FieldFunctionSignatureChar").charAt(0));
    _signatureLabel.setLabelFor(_signatureCombo);
    _signatureLabel.setText(res.getString("FieldFunctionSignatureName"));
    _signatureCombo.setMinimumSize(new Dimension(126, 21));
    _signatureCombo.setPreferredSize(new Dimension(130, 21));
    _signatureCombo.setToolTipText(res.getString("FieldFunctionSignatureTip"));
    _signatureCombo.setEditable(false);
    _descriptionLabel.setDisplayedMnemonic(res.getString("FieldFunctionDescriptionChar").charAt(0));
    _descriptionLabel.setLabelFor(_descriptionField);
    _descriptionLabel.setText(res.getString("FieldFunctionDescriptionName"));
    _descriptionScroll.setBorder(_textBorder);
    _descriptionScroll.setPreferredSize(new Dimension(200, 40));
    _descriptionField.setBorder(null);
    _descriptionField.setToolTipText(res.getString("FieldFunctionDescriptionTip"));
    _descriptionField.setFont(new java.awt.Font("SansSerif", 0, 12));
    _exampleLabel.setDisplayedMnemonic(res.getString("FieldFunctionExampleChar").charAt(0));
    _exampleLabel.setLabelFor(_exampleField);
    _exampleLabel.setText(res.getString("FieldFunctionExampleName"));
    _exampleScroll.setBorder(_textBorder);
    _exampleScroll.setPreferredSize(new Dimension(200, 40));
    _exampleField.setBorder(null);
    _exampleField.setToolTipText(res.getString("FieldFunctionExampleTip"));
    _exampleField.setFont(new java.awt.Font("SansSerif", 0, 12));
    this.add(_bodyPanel, BorderLayout.CENTER);
    _bodyPanel.setLayout(_gridBagLayout);

    _bodyPanel.add(_functionLabel,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_functionCombo,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_addFunctionButton,  new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_displayLabel,  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_displayField,   new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_deleteFunctionButton,  new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_smallIconLabel,  new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_smallIconField,   new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_largeIconLabel,  new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_largeIconField,   new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_classLabel,  new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_classField,   new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_classButton,  new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_signatureLabel,  new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_signatureCombo,   new GridBagConstraints(1, 5, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_descriptionLabel,  new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_descriptionScroll,   new GridBagConstraints(1, 6, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 2), 0, 0));
    _bodyPanel.add(_exampleLabel,  new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_exampleScroll,   new GridBagConstraints(1, 7, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 2), 0, 0));
    _descriptionScroll.getViewport().add(_descriptionField, null);
    _exampleScroll.getViewport().add(_exampleField, null);
	}

	/**
	 * Initialise UI.
	 */
	private void initialize() {
		updateControls();
	}

  /**
   * Synchronise controls with field contents and JSP version.
   */
  protected void updateControls() {
    boolean enabled = (_functionModel.getSize() > 0);
    _deleteFunctionButton.setEnabled(enabled);
    _displayLabel.setEnabled(enabled);
    _displayField.setEnabled(enabled);
    _displayField.setBackground(getFieldBackground(enabled));
    _smallIconLabel.setEnabled(enabled);
    _smallIconField.setEnabled(enabled);
    _smallIconField.setBackground(getFieldBackground(enabled));
    _largeIconLabel.setEnabled(enabled);
    _largeIconField.setEnabled(enabled);
    _largeIconField.setBackground(getFieldBackground(enabled));
    _classLabel.setEnabled(enabled);
    _classField.setEnabled(enabled);
    _classField.setBackground(getFieldBackground(enabled));
    _classButton.setEnabled(enabled);
    _signatureLabel.setEnabled(enabled);
    _signatureCombo.setEnabled(enabled);
    _descriptionLabel.setEnabled(enabled);
    _descriptionField.setEnabled(enabled);
    _descriptionField.setBackground(getFieldBackground(enabled));
    _exampleLabel.setEnabled(enabled);
    _exampleField.setEnabled(enabled);
    _exampleField.setBackground(getFieldBackground(enabled));
	}

	/**
	 * Check for user input error.
	 *
	 * The user has completed this page so check for input error.  If any, then
	 * report the error in a dialog and throw an exception.
   * 
   * @throws  VetoException  if not valid
	 */
	public void checkPage() throws VetoException {
    // Ensure any changes to the fields are saved
    _functionCombo_itemStateChanged(null);
    int index = -1;
    try {
      for (index = 0; index < _functionModel.getSize(); index++) {
        TaglibDescriptorFunction function = 
            (TaglibDescriptorFunction)_functionModel.getElementAt(index);
        // Each function must have a class name and method signature
        checkCondition(function.getFunctionClass().length() > 0,
            res.getString("FunctionClassMandatory"), _classField);
        checkCondition(function.getFunctionSig().length() > 0,
            res.getString("FunctionSignatureMandatory"), _signatureCombo);
      }
    }
    catch (VetoException ve) {
      _functionCombo.setSelectedIndex(index);
      throw ve;
    }
	}

  /**
   * Populate fields from selected function.
   * 
   * @param  e  the triggering event
   */
  void _functionCombo_itemStateChanged(ItemEvent e) {
    if (_lastFunction != null) {
      _lastFunction.setDisplayName(_displayField.getText());
      _lastFunction.setSmallIcon(_smallIconField.getText());
      _lastFunction.setLargeIcon(_largeIconField.getText());
      _lastFunction.setFunctionClass(_classField.getText());
      _lastFunction.setFunctionSig((String)_signatureCombo.getSelectedItem());
      _lastFunction.setDescription(_descriptionField.getText());
      _lastFunction.setExample(_exampleField.getText());
    }
    _lastFunction = (TaglibDescriptorFunction)_functionCombo.getSelectedItem();
    _displayField.setText(
        _lastFunction == null ? "" : _lastFunction.getDisplayName());
    _smallIconField.setText(
        _lastFunction == null ? "" : _lastFunction.getSmallIcon());
    _largeIconField.setText(
        _lastFunction == null ? "" : _lastFunction.getLargeIcon());
    _classField.setText(
        _lastFunction == null ? "" : _lastFunction.getFunctionClass());
    loadMethods(_classField.getText());
    _signatureCombo.setSelectedItem(
        _lastFunction == null ? "" : _lastFunction.getFunctionSig());
    _descriptionField.setText(
        _lastFunction == null ? "" : _lastFunction.getDescription());
    _exampleField.setText(
        _lastFunction == null ? "" : _lastFunction.getExample());
  }

  /**
   * Add a new function to the list.
   * 
   * @param  e  the triggering event
   */
  void _addFunctionButton_actionPerformed(ActionEvent e) {
    String name = JOptionPane.showInputDialog(
        this, res.getString("FunctionNamePrompt"), 
        getPageTitle(), JOptionPane.QUESTION_MESSAGE);
    if (name != null) {
      for (int index = 0; index < _functionModel.getSize(); index++) {
        TaglibDescriptorFunction function = 
            (TaglibDescriptorFunction)_functionModel.getElementAt(index);
        if (name.equals(function.getName())) {
          _functionCombo.setSelectedIndex(index);
          updateControls();
          return;
        }
      }
      _functionModel.addElement(new TaglibDescriptorFunction(name));
      _functionCombo.setSelectedIndex(_functionModel.getSize() - 1);
      _displayField.requestFocus();
      updateControls();
    }
  }

  /**
   * Delete a function from the list.
   * 
   * @param  e  the triggering event
   */
  void _deleteFunctionButton_actionPerformed(ActionEvent e) {
    _lastFunction = null;
    _functionModel.removeElement(_functionCombo.getSelectedItem());
    _functionCombo.setSelectedIndex(_functionModel.getSize() - 1);
    _displayField.requestFocus();
    updateControls();
  }

  /**
   * Select the class for this function.
   * 
   * @param  e  the triggering event
   */
  void _classButton_actionPerformed(ActionEvent e) {
    String className = PackageBrowserTree.browseClass(
        (JBProject)wizardHost.getBrowser().getActiveProject(), this, 
        res.getString("ChooseFunctionClassTitle"), _classField.getText());
    if (className == null) {
      return;
    }
    _classField.setText(className);
    loadMethods(className);
  }

  /**
   * Load the signature combobox with the methods from a class.
   * 
   * @param  className  the name of the class
   */
  private void loadMethods(String className) {
    _signatureModel.removeAllElements();
    if (className.length() == 0) {
      return;
    }
    try {
      // Load the class
      Class funcClass = _wizard.getClassLoader().loadClass(className);
      Method[] methods = funcClass.getMethods();
      ArrayList signatures = new ArrayList();
      // Find all public methods that return a value
      for (int index = 0; index < methods.length; index++) {
        Method method = methods[index];
        if (Modifier.isPublic(method.getModifiers()) && 
            !method.getReturnType().getName().equals("void")) {
          StringBuffer buffer =
              new StringBuffer(decodeType(method.getReturnType().getName())).
              append(" ").append(method.getName()).append("(");
          Class[] params = method.getParameterTypes();
          for (int index2 = 0; index2 < params.length; index2++) {
            buffer.append(index2 > 0 ? ", " : "").
                append(decodeType(params[index2].getName()));
          }
          signatures.add(buffer.append(")").toString());
        }
      }
      if (signatures.size() == 0) {
        // No public methods that return a value
        JOptionPane.showMessageDialog(this, 
            res.getString("NoPublicMethodsError"), 
            getPageTitle(), JOptionPane.ERROR_MESSAGE);
      }
      // Sort by method name
      Collections.sort(signatures, SIG_COMPARATOR);
      for (int index = 0; index < signatures.size(); index++) {
        _signatureModel.addElement(signatures.get(index));
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
      JOptionPane.showMessageDialog(this, ex.getClass().getName() + "\n" + 
          ex.getMessage(), getPageTitle(), JOptionPane.ERROR_MESSAGE);
    }
    if (_signatureModel.getSize() == 0) {
      _signatureModel.addElement("");
    }
    _signatureCombo.setSelectedIndex(0);
  }
  
  /**
   * Decode a type signature into normal text.
   * 
   * @param  type  the encoded type signature
   * @return  the human-readable version
   */
  private String decodeType(String type) {
    if (type.charAt(0) != '[') {
      return type;
    }
    StringBuffer signature = new StringBuffer();
    int index = 0;
    while (type.charAt(index) == '[') {
      signature.append("[]");
      index++;
    }
    switch (type.charAt(index)) {
      case 'B': signature.insert(0, "byte");    break;
      case 'C': signature.insert(0, "char");    break;
      case 'D': signature.insert(0, "double");  break;
      case 'F': signature.insert(0, "float");   break;
      case 'I': signature.insert(0, "int");     break;
      case 'J': signature.insert(0, "long");    break;
      case 'L': signature.insert(0, 
                    type.substring(index + 1, type.length() - 1)); break;
      case 'S': signature.insert(0, "short");   break;
      case 'Z': signature.insert(0, "boolean"); break;
      default:  signature.insert(0, type.substring(index));
    }
    return signature.toString();
  }

  /**
   * Fetches the set of functions for the taglib.
   *
   * @return  the function details
   */
  public ArrayList getFunctions() {
    ArrayList functions = new ArrayList();
    for (int index = 0; index < _functionModel.getSize(); index++) {
      functions.add(_functionModel.getElementAt(index));
    }
    return functions;
  }
}
