package wood.keith.opentools.wizards.jsptags;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.util.ArrayList;
import javax.servlet.jsp.tagext.TagExtraInfo;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

import com.borland.jbuilder.ide.PackageBrowserTree;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.wizard.WizardHost;

/**
 * This class contains the wizard page which allows the user to specify
 * details about each tag and its attributes.
 *
 * @author   Keith Wood
 * @version  1.0  9 November 2000
 * @version  2.0  6 February 2002 - JSP 1.2
 * @version  3.0  10 February 2004 - JSP 2.0
 */
public class TaglibDescriptorWizardPage6 extends TaglibDescriptorWizardPage {

  // Internal variables
  private ArrayList _tags;
  private TaglibDescriptorTag _tag;
  private TaglibDescriptorAttribute _attribute = null;
  private TaglibDescriptorVariable _variable = null;

  // UI variables
  private BorderLayout _borderLayout = new BorderLayout();
  private JPanel _bodyPanel = new JPanel();
  private GridBagLayout _gridBagLayout = new GridBagLayout();
  private JLabel _jLabel1 = new JLabel();
  private JComboBox _tagClassCombo = new JComboBox();
  private JLabel _jLabel2 = new JLabel();
  private JTextField _tagNameField = new JTextField();
  private JTabbedPane _detailsTabs = new JTabbedPane();
  private JPanel _standardPanel = new JPanel();
  private GridBagLayout _gridBagStandard = new GridBagLayout();
  private ButtonGroup _contentGroup = new ButtonGroup();
  private JLabel _jLabel3 = new JLabel();
  private JPanel _contentPanel = new JPanel();
  private FlowLayout _flowLayout = new FlowLayout();
  private JRadioButton _emptyRadio = new JRadioButton();
  private JRadioButton _jspRadio = new JRadioButton();
  private JRadioButton _dependentRadio = new JRadioButton();
  private JRadioButton _scriptlessRadio = new JRadioButton();
  private JLabel _jLabel4 = new JLabel();
  private JScrollPane _infoScroll = new JScrollPane();
  private JTextArea _infoField = new JTextArea();
  private JLabel _extraInfoLabel = new JLabel();
  private JComboBox _extraInfoCombo = new JComboBox();
  private JButton _extraInfoButton = new JButton();
  private JPanel _attributesPanel = new JPanel();
  private GridBagLayout _gridBagAttributes = new GridBagLayout();
  private JLabel _attributesLabel = new JLabel();
  private JComboBox _attributeCombo = new JComboBox();
  private JCheckBox _requiredCheckBox = new JCheckBox();
  private JCheckBox _expressionCheckBox = new JCheckBox();
  private JCheckBox _includeCheckBox = new JCheckBox();
  private JCheckBox _fragmentCheckBox = new JCheckBox();
  private JLabel _attrDescLabel = new JLabel();
  private JScrollPane _attrDescScroll = new JScrollPane();
  private JTextArea _attrDescField = new JTextArea();
  private JPanel _toolsPanel = new JPanel();
  private GridBagLayout _gridBagTools = new GridBagLayout();
  private JLabel _displayNameLabel = new JLabel();
  private JTextField _displayNameField = new JTextField();
  private JLabel _smallIconLabel = new JLabel();
  private JTextField _largeIconField = new JTextField();
  private JLabel _largeIconLabel = new JLabel();
  private JTextField _smallIconField = new JTextField();
  private JLabel _jLabel5 = new JLabel();
  private JScrollPane _exampleScroll = new JScrollPane();
  private JTextArea _exampleField = new JTextArea();
  private JPanel _variablesPanel = new JPanel();
  private GridBagLayout _gridBagVariables = new GridBagLayout();
  private JLabel _jLabel7 = new JLabel();
  private JComboBox _variableCombo = new JComboBox();
  private JButton _addVariableButton = new JButton();
  private JButton _removeVariableButton = new JButton();
  private JCheckBox _fromAttributeCheckBox = new JCheckBox();
  private JCheckBox _declaredCheckBox = new JCheckBox();
  private JLabel _varClassLabel = new JLabel();
  private JTextField _varClassField = new JTextField();
  private JButton _varClassButton = new JButton();
  private JLabel _attrTypeLabel = new JLabel();
  private JTextField _attrTypeField = new JTextField();
  private JButton _attrClassButton = new JButton();
  private JLabel _varScopeLabel = new JLabel();
  private JComboBox _varScopeCombo = new JComboBox();
  private JLabel _varDescLabel = new JLabel();
  private JScrollPane _varDescScroll = new JScrollPane();
  private JTextArea _varDescField = new JTextArea();
  private Border _tabBorder;
  private Border _textBorder;

  /**
   * Create a new wizard page that deals with tags and their attributes.
   *
   * @param  wizard         the wizard invoking this page
   * @param  tags           the list of TaglibDescriptorTag objects for display
   * @param  tagExtraInfos  the list of TagExtraInfo objects for selection
   */
  public TaglibDescriptorWizardPage6(TaglibDescriptorWizard wizard,
      ArrayList tags, ArrayList tagExtraInfos) {
    super(wizard, 6);
    _tags = tags;
    try {
      jbInit();
      loadTags(tagExtraInfos);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**
   * Define the user interface.
   *
   * The user interface is constructed here so the JBuilder UI Designer
   * may be used to edit it.
   *
   * @throws  Exception  if an error arises
   */
  protected void jbInit() throws Exception {
    _tabBorder = BorderFactory.createEmptyBorder(4,4,4,4);
    _textBorder = BorderFactory.createEtchedBorder(SystemColor.controlLtHighlight,SystemColor.controlShadow);
    this.setPageTitle(res.getString("TLDPage6Title"));
    try {
      this.setLargeIcon(_icon);
    }
    catch (Exception ex) {
      // Ignore
    }
    this.setInstructions(res.getString("TLDPage6Instructions"));
    this.setLayout(_borderLayout);

    _bodyPanel.setLayout(_gridBagLayout);
    _jLabel1.setText(res.getString("FieldTagClassName"));
    _jLabel1.setDisplayedMnemonic(res.getString("FieldTagClassChar").charAt(0));
    _jLabel1.setLabelFor(_tagClassCombo);
    _tagClassCombo.setMinimumSize(new Dimension(126, 21));
    _tagClassCombo.setPreferredSize(new Dimension(130, 21));
    _tagClassCombo.setToolTipText(res.getString("FieldTagClassTip"));
    _tagClassCombo.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _tagClassCombo_itemStateChanged(e);
      }
    });
    _jLabel2.setDisplayedMnemonic(res.getString("FieldTagNameChar").charAt(0));
    _jLabel2.setLabelFor(_tagNameField);
    _jLabel2.setText(res.getString("FieldTagNameName"));
    _tagNameField.setToolTipText(res.getString("FieldTagNameTip"));
    _standardPanel.setLayout(_gridBagStandard);
    _standardPanel.setBorder(_tabBorder);
    _jLabel3.setText(res.getString("FieldBodyContentName"));
    _jLabel3.setDisplayedMnemonic('0');
    _contentPanel.setLayout(_flowLayout);
    _contentPanel.setPreferredSize(new Dimension(237, 21));
    _contentPanel.setMinimumSize(new Dimension(237, 21));
    _flowLayout.setVgap(0);
    _emptyRadio.setMnemonic(res.getString("FieldContentEmptyChar").charAt(0));
    _emptyRadio.setToolTipText(res.getString("FieldContentEmptyTip"));
    _emptyRadio.setText(res.getString("FieldContentEmptyName"));
    _emptyRadio.setMargin(new Insets(0, 0, 0, 0));
    _jspRadio.setMnemonic(res.getString("FieldContentJspChar").charAt(0));
    _jspRadio.setToolTipText(res.getString("FieldContentJspTip"));
    _jspRadio.setText(res.getString("FieldContentJspName"));
    _jspRadio.setMargin(new Insets(0, 0, 0, 0));
    _dependentRadio.setMnemonic(res.getString("FieldContentDependentChar").charAt(0));
    _dependentRadio.setToolTipText(res.getString("FieldContentDependentTip"));
    _dependentRadio.setText(res.getString("FieldContentDependentName"));
    _dependentRadio.setMargin(new Insets(0, 0, 0, 0));
    _scriptlessRadio.setMnemonic(res.getString("FieldContentScriptlessChar").charAt(0));
    _scriptlessRadio.setToolTipText(res.getString("FieldContentScriptlessTip"));
    _scriptlessRadio.setText(res.getString("FieldContentScriptlessName"));
    _scriptlessRadio.setMargin(new Insets(0, 0, 0, 0));
    _jLabel4.setText(res.getString("FieldTagDescriptionName"));
    _jLabel4.setDisplayedMnemonic(res.getString("FieldTagDescriptionChar").charAt(0));
    _jLabel4.setLabelFor(_infoField);
    _infoScroll.setBorder(_textBorder);
    _infoField.setBorder(null);
    _infoField.setToolTipText(res.getString("FieldTagDescriptionTip"));
    _infoField.setFont(new java.awt.Font("SansSerif", 0, 12));
    _extraInfoLabel.setText(res.getString("FieldExtraInfoName"));
    _extraInfoLabel.setLabelFor(_extraInfoCombo);
    _extraInfoLabel.setDisplayedMnemonic(res.getString("FieldExtraInfoChar").charAt(0));
    _extraInfoCombo.setEditable(false);
    _extraInfoCombo.setPreferredSize(new Dimension(130, 21));
    _extraInfoCombo.setToolTipText(res.getString("FieldExtraInfoTip"));
    _extraInfoCombo.setMinimumSize(new Dimension(126, 21));
    _extraInfoButton.setMaximumSize(new Dimension(79, 21));
    _extraInfoButton.setMinimumSize(new Dimension(51, 21));
    _extraInfoButton.setPreferredSize(new Dimension(51, 21));
    _extraInfoButton.setMargin(new Insets(2, 4, 2, 4));
    _extraInfoButton.setMnemonic(res.getString("ButtonExtraInfoChar").charAt(0));
    _extraInfoButton.setText(res.getString("ButtonExtraInfoName"));
    _extraInfoButton.setToolTipText(res.getString("ButtonExtraInfoTip"));
    _extraInfoButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _extraInfoButton_actionPerformed(e);
      }
    });
    _attributesPanel.setLayout(_gridBagAttributes);
    _attributesPanel.setBorder(_tabBorder);
    _attributesLabel.setDisplayedMnemonic(res.getString("FieldAttributeChar").charAt(0));
    _attributesLabel.setLabelFor(_attributeCombo);
    _attributesLabel.setText(res.getString("FieldAttributeName"));
    _attributeCombo.setMinimumSize(new Dimension(126, 21));
    _attributeCombo.setPreferredSize(new Dimension(130, 21));
    _attributeCombo.setToolTipText(res.getString("FieldAttributeTip"));
    _attributeCombo.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _attributeCombo_itemStateChanged(e);
      }
    });
    _attrTypeLabel.setDisplayedMnemonic(res.getString("FieldAttributeTypeChar").charAt(0));
    _attrTypeLabel.setLabelFor(_attrTypeField);
    _attrTypeLabel.setText(res.getString("FieldAttributeTypeName"));
    _attrTypeField.setToolTipText(res.getString("FieldAttributeTypeTip"));
    _attrClassButton.setMinimumSize(new Dimension(51, 21));
    _attrClassButton.setPreferredSize(new Dimension(51, 21));
    _attrClassButton.setToolTipText(res.getString("ButtonAttributeTypeTip"));
    _attrClassButton.setMargin(new Insets(2, 4, 2, 4));
    _attrClassButton.setMnemonic(res.getString("ButtonAttributeTypeChar").charAt(0));
    _attrClassButton.setText(res.getString("ButtonAttributeTypeName"));
    _attrClassButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _attrClassButton_actionPerformed(e);
      }
    });
    _requiredCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _requiredCheckBox.setText(res.getString("FieldAttributeRequiredName"));
    _requiredCheckBox.setToolTipText(res.getString("FieldAttributeRequiredTip"));
    _requiredCheckBox.setMnemonic(res.getString("FieldAttributeRequiredChar").charAt(0));
    _requiredCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    _expressionCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _expressionCheckBox.setText(res.getString("FieldAttributeExpressionName"));
    _expressionCheckBox.setToolTipText(res.getString("FieldAttributeExpressionTip"));
    _expressionCheckBox.setMnemonic(res.getString("FieldAttributeExpressionChar").charAt(0));
    _includeCheckBox.setText(res.getString("FieldAttributeIncludedName"));
    _includeCheckBox.setToolTipText(res.getString("FieldAttributeIncludedTip"));
    _includeCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _includeCheckBox.setMnemonic(res.getString("FieldAttributeIncludedChar").charAt(0));
    _fragmentCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _fragmentCheckBox.setText(res.getString("FieldAttributeFragmentName"));
    _fragmentCheckBox.setToolTipText(res.getString("FieldAttributeFragmentTip"));
    _fragmentCheckBox.setMnemonic(res.getString("FieldAttributeFragmentChar").charAt(0));
    _fragmentCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    _fragmentCheckBox.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _fragmentCheckBox_itemStateChanged(e);
      }
    });
    _attrDescLabel.setDisplayedMnemonic(res.getString("FieldAttributeDescriptionChar").charAt(0));
    _attrDescLabel.setLabelFor(_attrDescField);
    _attrDescLabel.setText(res.getString("FieldAttributeDescriptionName"));
    _attrDescScroll.setBorder(_textBorder);
    _attrDescField.setToolTipText(res.getString("FieldAttributeDescriptionTip"));
    _attrDescField.setFont(new java.awt.Font("SansSerif", 0, 12));
    _toolsPanel.setLayout(_gridBagTools);
    _toolsPanel.setBorder(_tabBorder);
    _displayNameLabel.setDisplayedMnemonic(res.getString("FieldTagDisplayNameChar").charAt(0));
    _displayNameLabel.setLabelFor(_displayNameField);
    _displayNameLabel.setText(res.getString("FieldTagDisplayNameName"));
    _displayNameField.setToolTipText(res.getString("FieldTagDisplayNameTip"));
    _smallIconLabel.setDisplayedMnemonic(res.getString("FieldTagSmallIconChar").charAt(0));
    _smallIconLabel.setLabelFor(_smallIconField);
    _smallIconLabel.setText(res.getString("FieldTagSmallIconName"));
    _smallIconField.setToolTipText(res.getString("FieldTagSmallIconTip"));
    _largeIconLabel.setDisplayedMnemonic(res.getString("FieldTagLargeIconChar").charAt(0));
    _largeIconLabel.setLabelFor(_largeIconField);
    _largeIconLabel.setText(res.getString("FieldTagLargeIconName"));
    _largeIconField.setToolTipText(res.getString("FieldTagLargeIconTip"));
    _jLabel5.setDisplayedMnemonic(res.getString("FieldTagExampleChar").charAt(0));
    _jLabel5.setLabelFor(_exampleField);
    _jLabel5.setText(res.getString("FieldTagExampleName"));
    _exampleScroll.setBorder(_textBorder);
    _exampleField.setToolTipText(res.getString("FieldTagExampleTip"));
    _exampleField.setFont(new java.awt.Font("SansSerif", 0, 12));
    _variablesPanel.setLayout(_gridBagVariables);
    _variablesPanel.setBorder(_tabBorder);
    _jLabel7.setDisplayedMnemonic(res.getString("FieldVariableChar").charAt(0));
    _jLabel7.setLabelFor(_variableCombo);
    _jLabel7.setText(res.getString("FieldVariableName"));
    _variableCombo.setToolTipText(res.getString("FieldVariableTip"));
    _variableCombo.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        _variableCombo_itemStateChanged(e);
      }
    });
    _addVariableButton.setMaximumSize(new Dimension(79, 21));
    _addVariableButton.setMinimumSize(new Dimension(35, 21));
    _addVariableButton.setPreferredSize(new Dimension(35, 21));
    _addVariableButton.setMargin(new Insets(2, 4, 2, 4));
    _addVariableButton.setMnemonic(res.getString("ButtonAddVariableChar").charAt(0));
    _addVariableButton.setText(res.getString("ButtonAddVariableName"));
    _addVariableButton.setToolTipText(res.getString("ButtonAddVariableTip"));
    _addVariableButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _addVariableButton_actionPerformed(e);
      }
    });
    _removeVariableButton.setText(res.getString("ButtonDeleteVariableName"));
    _removeVariableButton.setMnemonic(res.getString("ButtonDeleteVariableChar").charAt(0));
    _removeVariableButton.setToolTipText(res.getString("ButtonDeleteVariableTip"));
    _removeVariableButton.setMargin(new Insets(2, 4, 2, 4));
    _removeVariableButton.setPreferredSize(new Dimension(50, 21));
    _removeVariableButton.setMinimumSize(new Dimension(45, 21));
    _removeVariableButton.setMaximumSize(new Dimension(79, 21));
    _removeVariableButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _removeVariableButton_actionPerformed(e);
      }
    });
    _fromAttributeCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _fromAttributeCheckBox.setText(res.getString("FieldVariableIsAttributeName"));
    _fromAttributeCheckBox.setToolTipText(res.getString("FieldVariableIsAttributeTip"));
    _fromAttributeCheckBox.setMnemonic(res.getString("FieldVariableIsAttributeChar").charAt(0));
    _declaredCheckBox.setMargin(new Insets(0, 0, 0, 0));
    _declaredCheckBox.setText(res.getString("FieldVariableDeclaredName"));
    _declaredCheckBox.setToolTipText(res.getString("FieldVariableDeclaredTip"));
    _declaredCheckBox.setMnemonic(res.getString("FieldVariableDeclaredChar").charAt(0));
    _declaredCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    _varClassLabel.setDisplayedMnemonic(res.getString("FieldVariableClassChar").charAt(0));
    _varClassLabel.setLabelFor(_varClassField);
    _varClassLabel.setText(res.getString("FieldVariableClassName"));
    _varClassField.setToolTipText(res.getString("FieldVariableClassTip"));
    _varClassButton.setMinimumSize(new Dimension(43, 21));
    _varClassButton.setPreferredSize(new Dimension(43, 21));
    _varClassButton.setToolTipText(res.getString("ButtonVariableClassTip"));
    _varClassButton.setMargin(new Insets(2, 4, 2, 4));
    _varClassButton.setMnemonic(res.getString("ButtonVariableClassChar").charAt(0));
    _varClassButton.setText(res.getString("ButtonVariableClassName"));
    _varClassButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _varClassButton_actionPerformed(e);
      }
    });
    _varScopeLabel.setToolTipText("");
    _varScopeLabel.setDisplayedMnemonic(res.getString("FieldVariableScopeChar").charAt(0));
    _varScopeLabel.setLabelFor(_varScopeCombo);
    _varScopeLabel.setText(res.getString("FieldVariableScopeName"));
    _varScopeCombo.setToolTipText(res.getString("FieldVariableScopeTip"));
    for (int index = 0; index < TaglibDescriptorVariable.SCOPE_VALUES.length;
        index++) {
      _varScopeCombo.addItem(TaglibDescriptorVariable.SCOPE_VALUES[index]);
    }
    _varDescLabel.setDisplayedMnemonic(res.getString("FieldVariableDescriptionChar").charAt(0));
    _varDescLabel.setLabelFor(_varDescField);
    _varDescLabel.setText(res.getString("FieldVariableDescriptionName"));
    _varDescScroll.setBorder(_textBorder);
    _varDescField.setToolTipText(res.getString("FieldVariableDescriptionTip"));
    _varDescField.setFont(new java.awt.Font("SansSerif", 0, 12));
    this.add(_bodyPanel, BorderLayout.CENTER);
    _bodyPanel.add(_jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_tagClassCombo, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _bodyPanel.add(_jLabel2, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 2, 0), 0, 0));
    _bodyPanel.add(_tagNameField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _bodyPanel.add(_detailsTabs, new GridBagConstraints(0, 2, 3, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    _detailsTabs.add(_standardPanel, res.getString("StandardPanelTitle"));
    _standardPanel.add(_jLabel3, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _standardPanel.add(_contentPanel,  new GridBagConstraints(1, 0, GridBagConstraints.REMAINDER, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _contentPanel.add(_emptyRadio, null);
    _contentPanel.add(_jspRadio, null);
    _contentPanel.add(_dependentRadio, null);
    _contentPanel.add(_scriptlessRadio, null);
    _standardPanel.add(_jLabel4, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _standardPanel.add(_infoScroll,    new GridBagConstraints(1, 1, GridBagConstraints.REMAINDER, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 2, 0), 0, 0));
    _standardPanel.add(_extraInfoLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _standardPanel.add(_extraInfoCombo, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 0, 0), 0, 0));
    _standardPanel.add(_extraInfoButton, new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 2, 0), 0, 0));
    _infoScroll.getViewport().add(_infoField, null);
    _detailsTabs.add(_attributesPanel, res.getString("AttributesPanelTitle"));
    _attributesPanel.add(_attributesLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _attributesPanel.add(_attributeCombo, new GridBagConstraints(1, 0, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _attributesPanel.add(_attrTypeLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _attributesPanel.add(_attrTypeField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _attributesPanel.add(_attrClassButton, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _attributesPanel.add(_includeCheckBox,  new GridBagConstraints(1, 2, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 2, 0), 0, 0));
    _attributesPanel.add(_requiredCheckBox,  new GridBagConstraints(1, 2, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 4, 2, 0), 0, 0));
    _attributesPanel.add(_expressionCheckBox, new GridBagConstraints(1, 3, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 2, 0), 0, 0));
    _attributesPanel.add(_fragmentCheckBox, new GridBagConstraints(1, 3, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 4, 2, 0), 0, 0));
    _attributesPanel.add(_attrDescLabel, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _attributesPanel.add(_attrDescScroll, new GridBagConstraints(1, 4, GridBagConstraints.REMAINDER, 1, 0.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 0, 0), 0, 0));
    _attrDescScroll.getViewport().add(_attrDescField, null);
    _detailsTabs.add(_toolsPanel, res.getString("ToolsPanelTitle"));
    _toolsPanel.add(_displayNameLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _toolsPanel.add(_displayNameField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _toolsPanel.add(_smallIconLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _toolsPanel.add(_smallIconField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _toolsPanel.add(_largeIconLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _toolsPanel.add(_largeIconField, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _toolsPanel.add(_jLabel5, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _toolsPanel.add(_exampleScroll, new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 0, 0), 0, 0));
    _exampleScroll.getViewport().add(_exampleField, null);
    _detailsTabs.add(_variablesPanel, res.getString("VariablesPanelTitle"));
    _variablesPanel.add(_jLabel7, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _variablesPanel.add(_variableCombo, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _variablesPanel.add(_addVariableButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 2, 0), 0, 0));
    _variablesPanel.add(_fromAttributeCheckBox, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 2, 0), 0, 0));
    _variablesPanel.add(_declaredCheckBox, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 4, 2, 0), 0, 0));
    _variablesPanel.add(_removeVariableButton, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 2, 0), 0, 0));
    _variablesPanel.add(_varClassLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _variablesPanel.add(_varClassField, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _variablesPanel.add(_varClassButton, new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 2, 0), 0, 0));
    _variablesPanel.add(_varScopeLabel, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _variablesPanel.add(_varScopeCombo, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 4, 2, 0), 0, 0));
    _variablesPanel.add(_varDescLabel, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    _variablesPanel.add(_varDescScroll, new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 4, 0, 0), 0, 0));
    _varDescScroll.getViewport().add(_varDescField, null);
    _contentGroup.add(_emptyRadio);
    _contentGroup.add(_jspRadio);
    _contentGroup.add(_dependentRadio);
    _contentGroup.add(_scriptlessRadio);
    _detailsTabs.setSelectedComponent(_standardPanel);
  }

  /**
   * Update UI for JSP version on activation.
   *
   * @param  host  the host component for this wizard
   */
  public void activated(WizardHost host) {
    super.activated(host);
    setupVariableTab(_variable != null);
  }

  /**
   * Load the tag and tag extra info classes into the combo box.
   *
   * @param  tagExtraInfos  the list of TagExtraInfo objects for selection
   */
  private void loadTags(ArrayList tagExtraInfos) {
    _extraInfoCombo.addItem("");
    if (tagExtraInfos != null) {
      for (int index = 0; index < tagExtraInfos.size(); index++) {
        _extraInfoCombo.addItem(tagExtraInfos.get(index));
      }
    }

    for (int index = 0; index < _tags.size(); index++) {
      _tagClassCombo.addItem(_tags.get(index));
    }
    _tagClassCombo.setSelectedIndex(0);
    loadTag();
  }

  /**
   * Synchronise controls with field contents and JSP version.
   */
  protected void updateControls() {
    boolean isJSP11 = _jspVersion.equals(JSPTagProperties.JSP_11);
    boolean isJSP20 = _jspVersion.equals(JSPTagProperties.JSP_20);
    _scriptlessRadio.setEnabled(isJSP20);
    _attrTypeLabel.setEnabled(!isJSP11);
    _attrTypeField.setEnabled(!isJSP11);
    _attrTypeField.setBackground(getFieldBackground(!isJSP11));
    _attrDescLabel.setEnabled(!isJSP11);
    _attrDescField.setEnabled(!isJSP11);
    _attrDescField.setBackground(getFieldBackground(!isJSP11));
    _fragmentCheckBox.setEnabled(isJSP20);
    if (_detailsTabs.indexOfComponent(_toolsPanel) > -1) {
      _detailsTabs.setEnabledAt(
          _detailsTabs.indexOfComponent(_toolsPanel), !isJSP11);
    }
    if (_detailsTabs.indexOfComponent(_variablesPanel) > -1) {
      _detailsTabs.setEnabledAt(
          _detailsTabs.indexOfComponent(_variablesPanel), !isJSP11);
    }
    if (isJSP11 && (_detailsTabs.getSelectedComponent() == _toolsPanel ||
        _detailsTabs.getSelectedComponent() == _variablesPanel)) {
      _detailsTabs.setSelectedComponent(_standardPanel);
    }
  }

  /**
   * Setup based on presence of variables for a tag.
   *
   * @param  enabled  true if a variable present, false otherwise
   */
  private void setupVariableTab(boolean enabled) {
    _removeVariableButton.setEnabled(enabled);
    _fromAttributeCheckBox.setEnabled(enabled);
    _declaredCheckBox.setEnabled(enabled);
    _varClassLabel.setEnabled(enabled);
    _varClassField.setEnabled(enabled);
    _varClassField.setBackground(getFieldBackground(enabled));
    _varClassButton.setEnabled(enabled);
    _varScopeLabel.setEnabled(enabled);
    _varScopeCombo.setEnabled(enabled);
    _varDescLabel.setEnabled(enabled);
    _varDescField.setEnabled(enabled);
    _varDescField.setBackground(getFieldBackground(enabled));
  }

  /**
   * Update details for selected tag class.
   *
   * @param  e  the triggering event
   */
  void _tagClassCombo_itemStateChanged(ItemEvent e) {
    saveTag();
    loadTag();
  }

  /**
   * Update UI with details of specified tag.
   */
  private void loadTag() {
    _tag = (TaglibDescriptorTag)_tagClassCombo.getSelectedItem();
    _tagNameField.setText(_tag.getName());
    _infoField.setText(_tag.getDescription());
    _infoField.select(0, 0);
    _emptyRadio.setSelected(
        (_tag.getBodyContent() == TaglibDescriptorTag.CONTENT_EMPTY));
    _jspRadio.setSelected(
        (_tag.getBodyContent() == TaglibDescriptorTag.CONTENT_JSP));
    _dependentRadio.setSelected(
        (_tag.getBodyContent() == TaglibDescriptorTag.CONTENT_TAG));
    _scriptlessRadio.setSelected(
        (_tag.getBodyContent() == TaglibDescriptorTag.CONTENT_SCRIPTLESS));
    _extraInfoCombo.setSelectedItem(_tag.getTagExtraInfoClass());
    if (_extraInfoCombo.getSelectedIndex() == -1) {
      _extraInfoCombo.setSelectedIndex(0);
    }

    _displayNameField.setText(_tag.getDisplayName());
    _smallIconField.setText(_tag.getSmallIcon());
    _largeIconField.setText(_tag.getLargeIcon());
    _exampleField.setText(_tag.getExample());

    loadAttributes();
    loadVariables();
  }

  /**
   * Load all attributes for this tag into the combo box.
   */
  private void loadAttributes() {
    _attribute = null;
    _attributeCombo.removeAllItems();
    for (int index = 0; index < _tag.getAttributes().size(); index++) {
      _attributeCombo.addItem(_tag.getAttributes().get(index));
    }
    boolean hasAttributes = (_tag.getAttributes().size() > 0);
    _detailsTabs.setEnabledAt(_detailsTabs.indexOfComponent(_attributesPanel),
        hasAttributes);
    if (hasAttributes) {
      _attributeCombo.setSelectedIndex(0);
      loadAttribute();
    }
    else if (_detailsTabs.getSelectedComponent() == _attributesPanel) {
      _detailsTabs.setSelectedComponent(_standardPanel);
    }
  }

  /**
   * Update UI with details of specified attribute.
   */
  private void loadAttribute() {
    _attribute = (TaglibDescriptorAttribute)_attributeCombo.getSelectedItem();
    if (_attribute == null) {
      return;
    }

    _attrTypeField.setText(_attribute.getType());
    _includeCheckBox.setSelected(_attribute.isIncluded());
    _requiredCheckBox.setSelected(_attribute.isRequired());
    _expressionCheckBox.setSelected(_attribute.isExpressionAllowed());
    _fragmentCheckBox.setSelected(_attribute.isFragment());
    _attrDescField.setText(_attribute.getDescription());
  }

  /**
   * Load all variables for this tag into the combo box.
   */
  private void loadVariables() {
    _variableCombo.removeAllItems();
    for (int index = 0; index < _tag.getVariables().size(); index++) {
      _variableCombo.addItem(_tag.getVariables().get(index));
    }
    if (_tag.getVariables().size() > 0) {
      _variableCombo.setSelectedIndex(0);
    }
    loadVariable();
  }

  /**
   * Update UI with details of specified variable.
   */
  private void loadVariable() {
    _variable = (TaglibDescriptorVariable)_variableCombo.getSelectedItem();
    boolean enabled = (_variable != null);
    setupVariableTab(enabled);
    if (!enabled) {
      return;
    }

    _fromAttributeCheckBox.setSelected(_variable.isFromAttribute());
    _declaredCheckBox.setSelected(_variable.isDeclared());
    _varClassField.setText(_variable.getClassName());
    _varScopeCombo.setSelectedItem(_variable.getScope());
    _varDescField.setText(_variable.getDescription());
  }

  /**
   * Save changes to the tag.
   */
  private void saveTag() {
    if (_tag == null) {
      return;
    }

    _tag.setName(_tagNameField.getText());
    _tag.setDescription(_infoField.getText());
    if (_emptyRadio.isSelected()) {
      _tag.setBodyContent(TaglibDescriptorTag.CONTENT_EMPTY);
    }
    else if (_jspRadio.isSelected()) {
      _tag.setBodyContent(TaglibDescriptorTag.CONTENT_JSP);
    }
    else if (_dependentRadio.isSelected()) {
      _tag.setBodyContent(TaglibDescriptorTag.CONTENT_TAG);
    }
    else if (_scriptlessRadio.isSelected()) {
      _tag.setBodyContent(TaglibDescriptorTag.CONTENT_SCRIPTLESS);
    }
    _tag.setTagExtraInfoClass((String)_extraInfoCombo.getSelectedItem());
    _tag.setDisplayName(_displayNameField.getText());
    _tag.setSmallIcon(_smallIconField.getText());
    _tag.setLargeIcon(_largeIconField.getText());
    _tag.setExample(_exampleField.getText());
    saveAttribute();
    saveVariable();
  }

  /**
   * Save changes to the attribute.
   */
  private void saveAttribute() {
    if (_attribute == null){
      return;
    }

    _attribute.setType(_attrTypeField.getText());
    _attribute.setIncluded(_includeCheckBox.isSelected());
    _attribute.setRequired(_requiredCheckBox.isSelected());
    _attribute.setExpressionAllowed(_expressionCheckBox.isSelected());
    _attribute.setFragment(_fragmentCheckBox.isSelected());
    _attribute.setDescription(_attrDescField.getText());
  }

  /**
   * Save changes to the variable.
   */
  private void saveVariable() {
    if (_variable == null){
      return;
    }

    _variable.setFromAttribute(_fromAttributeCheckBox.isSelected());
    _variable.setDeclared(_declaredCheckBox.isSelected());
    _variable.setClassName(_varClassField.getText());
    _variable.setScope((String)_varScopeCombo.getSelectedItem());
    _variable.setDescription(_varDescField.getText());
  }

  /**
   * Select the tag extra information class.
   *
   * @param  e  the triggering event
   */
  void _extraInfoButton_actionPerformed(ActionEvent e) {
    String className = PackageBrowserTree.browseClass(
        (JBProject)wizardHost.getBrowser().getActiveProject(), this,
        res.getString("ChooseExtraInfoClassTitle"),
        (String)_extraInfoCombo.getSelectedItem());
    if (className != null && className.length() > 0) {
      Class extraInfoClass = null;
      try {
        extraInfoClass = _wizard.getClassLoader().loadClass(className);
      }
      catch (ClassNotFoundException ex) {
        JOptionPane.showMessageDialog(this, 
            res.getString("NoClassFoundError"),
            getPageTitle(), JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (!TagExtraInfo.class.isAssignableFrom(extraInfoClass)) {
        JOptionPane.showMessageDialog(this,
            res.getString("NotATagExtraInfoObjectError"),
            getPageTitle(), JOptionPane.WARNING_MESSAGE);
      }
      if (((DefaultComboBoxModel)_extraInfoCombo.getModel()).
          getIndexOf(className) == -1) {
        _extraInfoCombo.addItem(className);
      }
      _extraInfoCombo.setSelectedItem(className);
      updateControls();
    }
  }

  /**
   * Update details for selected tag attribute.
   *
   * @param  e  the triggering event
   */
  void _attributeCombo_itemStateChanged(ItemEvent e) {
    saveAttribute();
    loadAttribute();
  }

  /**
   * Select the class for this attribute.
   *
   * @param  e  the triggering event
   */
  void _attrClassButton_actionPerformed(ActionEvent e) {
    String className = PackageBrowserTree.browseClass(
        (JBProject)wizardHost.getBrowser().getActiveProject(), this,
        res.getString("ChooseAttributeClassTitle"), _attrTypeField.getText());
    if (className != null && className.length() > 0) {
      _attrTypeField.setText(className);
    }
  }

  /**
   * Disable/enable dependent fields.
   *
   * @param  e  the triggering event
   */
  void _fragmentCheckBox_itemStateChanged(ItemEvent e) {
    _attrTypeLabel.setEnabled(!_fragmentCheckBox.isSelected());
    _attrTypeField.setEnabled(!_fragmentCheckBox.isSelected());
    _attrTypeField.setBackground(
        getFieldBackground(!_fragmentCheckBox.isSelected()));
    _attrClassButton.setEnabled(!_fragmentCheckBox.isSelected());
    _expressionCheckBox.setEnabled(!_fragmentCheckBox.isSelected());
  }

  /**
   * Update details for selected tag variable.
   *
   * @param  e  the triggering event
   */
  void _variableCombo_itemStateChanged(ItemEvent e) {
    saveVariable();
    loadVariable();
  }

  /**
   * Add a new variable to the list.
   *
   * @param  e  the triggering event
   */
  void _addVariableButton_actionPerformed(ActionEvent e) {
    String name = JOptionPane.showInputDialog(_bodyPanel, 
        res.getString("VariableNamePrompt"),
        TaglibDescriptorWizard.TITLE, JOptionPane.QUESTION_MESSAGE);
    if (name != null) {
      TaglibDescriptorVariable variable =
          new TaglibDescriptorVariable(name, true);
      _tag.getVariables().add(variable);
      _variableCombo.addItem(variable);
      _variableCombo.setSelectedItem(variable);
    }
    loadVariable();
  }

  /**
   * Remove the current variable from the list.
   *
   * @param  e  the triggering event
   */
  void _removeVariableButton_actionPerformed(ActionEvent e) {
    _tag.getVariables().remove(_variable);
    _variableCombo.removeItem(_variable);
    if (_variableCombo.getModel().getSize() > 0) {
      _variableCombo.setSelectedIndex(0);
    }
    loadVariable();
  }

  /**
   * Select the class for this variable.
   *
   * @param  e  the triggering event
   */
  void _varClassButton_actionPerformed(ActionEvent e) {
    String className = PackageBrowserTree.browseClass(
        (JBProject)wizardHost.getBrowser().getActiveProject(), this,
        res.getString("ChooseVariableClassTitle"), _varClassField.getText());
    if (className != null && className.length() > 0) {
      _varClassField.setText(className);
    }
  }

  /**
   * Check for user input error.
   *
   * The user has completed this page so check for input error.  If any, then
   * report the error in a dialog and throw an exception.
   *
   * @throws  VetoException  if not valid
   */
  public void checkPage() throws VetoException {
    // Save changes first
    saveAttribute();
    saveVariable();
    saveTag();
    for (int index = 0; index < _tags.size(); index++) {
      TaglibDescriptorTag tag = (TaglibDescriptorTag)_tags.get(index);
      checkCondition(tag.getName().length() > 0,
          res.getString("TagNameMandatory") + tag.getTagClassName(),
          _tagNameField);
    }
  }

  /**
   * Fetches the set of tags for the taglib.
   *
   * @return  the tag details
   */
  public ArrayList getTags() {
    return _tags;
  }
}
